@@ -21,9 +21,7 @@
 #include "qgsmeshterraingenerator.h"
 #include "qgsonlineterraingenerator.h"
 #include "qgsprojectviewsettings.h"
-#include "qgsvectorlayer3drenderer.h"
-#include "qgsmeshlayer3drenderer.h"
-#include "qgspointcloudlayer3drenderer.h"
+#include "qgsquantizedmeshterraingenerator.h"
 #include "qgsprojectelevationproperties.h"
 #include "qgsterrainprovider.h"
 #include "qgslightsource.h"
@@ -31,6 +29,9 @@
 #include "qgsrasterlayer.h"
 #include "qgspointlightsettings.h"
 #include "qgsdirectionallightsettings.h"
+#include "qgs3drendercontext.h"
+#include "qgsthreadingutils.h"
+#include "qgsmaplayerlistutils_p.h"
 
 #include <QDomDocument>
 #include <QDomElement>
@@ -120,6 +121,8 @@ Qgs3DMapSettings::~Qgs3DMapSettings()
 
 void Qgs3DMapSettings::readXml( const QDomElement &elem, const QgsReadWriteContext &context )
 {
+  QGIS_PROTECT_QOBJECT_THREAD_ACCESS
+
   QgsProjectDirtyBlocker blocker( QgsProject::instance() );
   QDomElement elemOrigin = elem.firstChildElement( QStringLiteral( "origin" ) );
   mOrigin = QgsVector3D(
@@ -261,6 +264,11 @@ void Qgs3DMapSettings::readXml( const QDomElement &elem, const QgsReadWriteConte
     meshTerrainGenerator->setCrs( mCrs, mTransformContext );
     setTerrainGenerator( meshTerrainGenerator );
   }
+  else if ( terrainGenType == QLatin1String( "quantizedmesh" ) )
+  {
+    QgsQuantizedMeshTerrainGenerator *qmTerrainGenerator = new QgsQuantizedMeshTerrainGenerator;
+    setTerrainGenerator( qmTerrainGenerator );
+  }
   else // "flat"
   {
     QgsFlatTerrainGenerator *flatGen = new QgsFlatTerrainGenerator;
@@ -317,6 +325,8 @@ void Qgs3DMapSettings::readXml( const QDomElement &elem, const QgsReadWriteConte
 
 QDomElement Qgs3DMapSettings::writeXml( QDomDocument &doc, const QgsReadWriteContext &context ) const
 {
+  QGIS_PROTECT_QOBJECT_THREAD_ACCESS
+
   QDomElement elem = doc.createElement( QStringLiteral( "qgis3d" ) );
 
   QDomElement elemOrigin = doc.createElement( QStringLiteral( "origin" ) );
@@ -452,6 +462,8 @@ QDomElement Qgs3DMapSettings::writeXml( QDomDocument &doc, const QgsReadWriteCon
 
 void Qgs3DMapSettings::resolveReferences( const QgsProject &project )
 {
+  QGIS_PROTECT_QOBJECT_THREAD_ACCESS
+
   for ( int i = 0; i < mLayers.count(); ++i )
   {
     QgsMapLayerRef &layerRef = mLayers[i];
@@ -459,10 +471,23 @@ void Qgs3DMapSettings::resolveReferences( const QgsProject &project )
   }
 
   mTerrainGenerator->resolveReferences( project );
+
+  // Set extent now that layer-based generators actually have a chance to know their CRS
+  QgsRectangle terrainExtent = Qgs3DUtils::tryReprojectExtent2D( mExtent, mCrs, mTerrainGenerator->crs(), mTransformContext );
+  mTerrainGenerator->setExtent( terrainExtent );
+}
+
+QgsRectangle Qgs3DMapSettings::extent() const
+{
+  QGIS_PROTECT_QOBJECT_THREAD_ACCESS
+
+  return mExtent;
 }
 
 void Qgs3DMapSettings::setExtent( const QgsRectangle &extent )
 {
+  QGIS_PROTECT_QOBJECT_THREAD_ACCESS
+
   if ( extent == mExtent )
     return;
 
@@ -477,33 +502,94 @@ void Qgs3DMapSettings::setExtent( const QgsRectangle &extent )
   emit extentChanged();
 }
 
+void Qgs3DMapSettings::setOrigin( const QgsVector3D &origin )
+{
+  QGIS_PROTECT_QOBJECT_THREAD_ACCESS
+
+  mOrigin = origin;
+}
+
+QgsVector3D Qgs3DMapSettings::origin() const
+{
+  QGIS_PROTECT_QOBJECT_THREAD_ACCESS
+
+  return mOrigin;
+}
+
 QgsVector3D Qgs3DMapSettings::mapToWorldCoordinates( const QgsVector3D &mapCoords ) const
 {
+  QGIS_PROTECT_QOBJECT_THREAD_ACCESS
+
   return Qgs3DUtils::mapToWorldCoordinates( mapCoords, mOrigin );
 }
 
 QgsVector3D Qgs3DMapSettings::worldToMapCoordinates( const QgsVector3D &worldCoords ) const
 {
+  QGIS_PROTECT_QOBJECT_THREAD_ACCESS
+
   return Qgs3DUtils::worldToMapCoordinates( worldCoords, mOrigin );
 }
 
 void Qgs3DMapSettings::setCrs( const QgsCoordinateReferenceSystem &crs )
 {
+  QGIS_PROTECT_QOBJECT_THREAD_ACCESS
+
   mCrs = crs;
 }
 
+QgsCoordinateReferenceSystem Qgs3DMapSettings::crs() const
+{
+  QGIS_PROTECT_QOBJECT_THREAD_ACCESS
+
+  return mCrs;
+}
+
 QgsCoordinateTransformContext Qgs3DMapSettings::transformContext() const
 {
+  QGIS_PROTECT_QOBJECT_THREAD_ACCESS
+
   return mTransformContext;
 }
 
 void Qgs3DMapSettings::setTransformContext( const QgsCoordinateTransformContext &context )
 {
+  QGIS_PROTECT_QOBJECT_THREAD_ACCESS
+
   mTransformContext = context;
 }
 
+const QgsPathResolver &Qgs3DMapSettings::pathResolver() const
+{
+  QGIS_PROTECT_QOBJECT_THREAD_ACCESS
+
+  return mPathResolver;
+}
+
+void Qgs3DMapSettings::setPathResolver( const QgsPathResolver &resolver )
+{
+  QGIS_PROTECT_QOBJECT_THREAD_ACCESS
+
+  mPathResolver = resolver;
+}
+
+QgsMapThemeCollection *Qgs3DMapSettings::mapThemeCollection() const
+{
+  QGIS_PROTECT_QOBJECT_THREAD_ACCESS
+
+  return mMapThemes;
+}
+
+void Qgs3DMapSettings::setMapThemeCollection( QgsMapThemeCollection *mapThemes )
+{
+  QGIS_PROTECT_QOBJECT_THREAD_ACCESS
+
+  mMapThemes = mapThemes;
+}
+
 void Qgs3DMapSettings::setBackgroundColor( const QColor &color )
 {
+  QGIS_PROTECT_QOBJECT_THREAD_ACCESS
+
   if ( color == mBackgroundColor )
     return;
 
@@ -513,11 +599,15 @@ void Qgs3DMapSettings::setBackgroundColor( const QColor &color )
 
 QColor Qgs3DMapSettings::backgroundColor() const
 {
+  QGIS_PROTECT_QOBJECT_THREAD_ACCESS
+
   return mBackgroundColor;
 }
 
 void Qgs3DMapSettings::setSelectionColor( const QColor &color )
 {
+  QGIS_PROTECT_QOBJECT_THREAD_ACCESS
+
   if ( color == mSelectionColor )
     return;
 
@@ -527,11 +617,15 @@ void Qgs3DMapSettings::setSelectionColor( const QColor &color )
 
 QColor Qgs3DMapSettings::selectionColor() const
 {
+  QGIS_PROTECT_QOBJECT_THREAD_ACCESS
+
   return mSelectionColor;
 }
 
 void Qgs3DMapSettings::setTerrainVerticalScale( double zScale )
 {
+  QGIS_PROTECT_QOBJECT_THREAD_ACCESS
+
   if ( zScale == mTerrainVerticalScale )
     return;
 
@@ -541,27 +635,28 @@ void Qgs3DMapSettings::setTerrainVerticalScale( double zScale )
 
 double Qgs3DMapSettings::terrainVerticalScale() const
 {
+  QGIS_PROTECT_QOBJECT_THREAD_ACCESS
+
   return mTerrainVerticalScale;
 }
 
 void Qgs3DMapSettings::setLayers( const QList<QgsMapLayer *> &layers )
 {
-  QList<QgsMapLayerRef> lst;
-  lst.reserve( layers.count() );
-  for ( QgsMapLayer *layer : layers )
-  {
-    lst.append( layer );
-  }
+  QGIS_PROTECT_QOBJECT_THREAD_ACCESS
 
-  if ( mLayers == lst )
+  const QList<QgsMapLayer *> raw = _qgis_listRefToRaw( mLayers );
+
+  if ( layers == raw )
     return;
 
-  mLayers = lst;
+  mLayers = _qgis_listRawToRef( layers );
   emit layersChanged();
 }
 
 QList<QgsMapLayer *> Qgs3DMapSettings::layers() const
 {
+  QGIS_PROTECT_QOBJECT_THREAD_ACCESS
+
   QList<QgsMapLayer *> lst;
   lst.reserve( mLayers.count() );
   for ( const QgsMapLayerRef &layerRef : mLayers )
@@ -574,6 +669,8 @@ QList<QgsMapLayer *> Qgs3DMapSettings::layers() const
 
 void Qgs3DMapSettings::configureTerrainFromProject( QgsProjectElevationProperties *properties, const QgsRectangle &fullExtent )
 {
+  QGIS_PROTECT_QOBJECT_THREAD_ACCESS
+
   setExtent( fullExtent );
   if ( properties->terrainProvider()->type() == QLatin1String( "flat" ) )
   {
@@ -620,6 +717,8 @@ void Qgs3DMapSettings::configureTerrainFromProject( QgsProjectElevationPropertie
 
 void Qgs3DMapSettings::setMapTileResolution( int res )
 {
+  QGIS_PROTECT_QOBJECT_THREAD_ACCESS
+
   if ( mMapTileResolution == res )
     return;
 
@@ -629,11 +728,15 @@ void Qgs3DMapSettings::setMapTileResolution( int res )
 
 int Qgs3DMapSettings::mapTileResolution() const
 {
+  QGIS_PROTECT_QOBJECT_THREAD_ACCESS
+
   return mMapTileResolution;
 }
 
 void Qgs3DMapSettings::setMaxTerrainScreenError( float error )
 {
+  QGIS_PROTECT_QOBJECT_THREAD_ACCESS
+
   if ( mMaxTerrainScreenError == error )
     return;
 
@@ -643,11 +746,15 @@ void Qgs3DMapSettings::setMaxTerrainScreenError( float error )
 
 float Qgs3DMapSettings::maxTerrainScreenError() const
 {
+  QGIS_PROTECT_QOBJECT_THREAD_ACCESS
+
   return mMaxTerrainScreenError;
 }
 
 void Qgs3DMapSettings::setMaxTerrainGroundError( float error )
 {
+  QGIS_PROTECT_QOBJECT_THREAD_ACCESS
+
   if ( mMaxTerrainGroundError == error )
     return;
 
@@ -657,34 +764,59 @@ void Qgs3DMapSettings::setMaxTerrainGroundError( float error )
 
 void Qgs3DMapSettings::setTerrainElevationOffset( float offset )
 {
+  QGIS_PROTECT_QOBJECT_THREAD_ACCESS
+
   if ( mTerrainElevationOffset == offset )
     return;
   mTerrainElevationOffset = offset;
   emit terrainElevationOffsetChanged( mTerrainElevationOffset );
 }
 
+float Qgs3DMapSettings::terrainElevationOffset() const
+{
+  QGIS_PROTECT_QOBJECT_THREAD_ACCESS
+
+  return mTerrainElevationOffset;
+}
+
 float Qgs3DMapSettings::maxTerrainGroundError() const
 {
+  QGIS_PROTECT_QOBJECT_THREAD_ACCESS
+
   return mMaxTerrainGroundError;
 }
 
 void Qgs3DMapSettings::setTerrainGenerator( QgsTerrainGenerator *gen )
 {
+  QGIS_PROTECT_QOBJECT_THREAD_ACCESS
+
   if ( mTerrainGenerator )
   {
     disconnect( mTerrainGenerator.get(), &QgsTerrainGenerator::terrainChanged, this, &Qgs3DMapSettings::terrainGeneratorChanged );
   }
 
-  QgsRectangle terrainExtent = Qgs3DUtils::tryReprojectExtent2D( mExtent, mCrs, gen->crs(), mTransformContext );
-  gen->setExtent( terrainExtent );
+  if ( gen->crs().isValid() ) // Don't bother setting an extent rect in the wrong CRS
+  {
+    QgsRectangle terrainExtent = Qgs3DUtils::tryReprojectExtent2D( mExtent, mCrs, gen->crs(), mTransformContext );
+    gen->setExtent( terrainExtent );
+  }
   mTerrainGenerator.reset( gen );
   connect( mTerrainGenerator.get(), &QgsTerrainGenerator::terrainChanged, this, &Qgs3DMapSettings::terrainGeneratorChanged );
 
   emit terrainGeneratorChanged();
 }
 
+QgsTerrainGenerator *Qgs3DMapSettings::terrainGenerator() const
+{
+  QGIS_PROTECT_QOBJECT_THREAD_ACCESS
+
+  return mTerrainGenerator.get();
+}
+
 void Qgs3DMapSettings::setTerrainShadingEnabled( bool enabled )
 {
+  QGIS_PROTECT_QOBJECT_THREAD_ACCESS
+
   if ( mTerrainShadingEnabled == enabled )
     return;
 
@@ -692,8 +824,17 @@ void Qgs3DMapSettings::setTerrainShadingEnabled( bool enabled )
   emit terrainShadingChanged();
 }
 
+bool Qgs3DMapSettings::isTerrainShadingEnabled() const
+{
+  QGIS_PROTECT_QOBJECT_THREAD_ACCESS
+
+  return mTerrainShadingEnabled;
+}
+
 void Qgs3DMapSettings::setTerrainShadingMaterial( const QgsPhongMaterialSettings &material )
 {
+  QGIS_PROTECT_QOBJECT_THREAD_ACCESS
+
   if ( mTerrainShadingMaterial == material )
     return;
 
@@ -701,8 +842,17 @@ void Qgs3DMapSettings::setTerrainShadingMaterial( const QgsPhongMaterialSettings
   emit terrainShadingChanged();
 }
 
+QgsPhongMaterialSettings Qgs3DMapSettings::terrainShadingMaterial() const
+{
+  QGIS_PROTECT_QOBJECT_THREAD_ACCESS
+
+  return mTerrainShadingMaterial;
+}
+
 void Qgs3DMapSettings::setTerrainMapTheme( const QString &theme )
 {
+  QGIS_PROTECT_QOBJECT_THREAD_ACCESS
+
   if ( mTerrainMapTheme == theme )
     return;
 
@@ -710,8 +860,17 @@ void Qgs3DMapSettings::setTerrainMapTheme( const QString &theme )
   emit terrainMapThemeChanged();
 }
 
+QString Qgs3DMapSettings::terrainMapTheme() const
+{
+  QGIS_PROTECT_QOBJECT_THREAD_ACCESS
+
+  return mTerrainMapTheme;
+}
+
 void Qgs3DMapSettings::setShowTerrainBoundingBoxes( bool enabled )
 {
+  QGIS_PROTECT_QOBJECT_THREAD_ACCESS
+
   if ( mShowTerrainBoundingBoxes == enabled )
     return;
 
@@ -719,8 +878,18 @@ void Qgs3DMapSettings::setShowTerrainBoundingBoxes( bool enabled )
   emit showTerrainBoundingBoxesChanged();
 }
 
+bool Qgs3DMapSettings::showTerrainBoundingBoxes() const
+{
+  QGIS_PROTECT_QOBJECT_THREAD_ACCESS
+
+  return mShowTerrainBoundingBoxes;
+}
+
+
 void Qgs3DMapSettings::setShowTerrainTilesInfo( bool enabled )
 {
+  QGIS_PROTECT_QOBJECT_THREAD_ACCESS
+
   if ( mShowTerrainTileInfo == enabled )
     return;
 
@@ -728,8 +897,17 @@ void Qgs3DMapSettings::setShowTerrainTilesInfo( bool enabled )
   emit showTerrainTilesInfoChanged();
 }
 
+bool Qgs3DMapSettings::showTerrainTilesInfo() const
+{
+  QGIS_PROTECT_QOBJECT_THREAD_ACCESS
+
+  return mShowTerrainTileInfo;
+}
+
 void Qgs3DMapSettings::setShowCameraViewCenter( bool enabled )
 {
+  QGIS_PROTECT_QOBJECT_THREAD_ACCESS
+
   if ( mShowCameraViewCenter == enabled )
     return;
 
@@ -737,8 +915,17 @@ void Qgs3DMapSettings::setShowCameraViewCenter( bool enabled )
   emit showCameraViewCenterChanged();
 }
 
+bool Qgs3DMapSettings::showCameraViewCenter() const
+{
+  QGIS_PROTECT_QOBJECT_THREAD_ACCESS
+
+  return mShowCameraViewCenter;
+}
+
 void Qgs3DMapSettings::setShowCameraRotationCenter( bool enabled )
 {
+  QGIS_PROTECT_QOBJECT_THREAD_ACCESS
+
   if ( mShowCameraRotationCenter == enabled )
     return;
 
@@ -746,9 +933,17 @@ void Qgs3DMapSettings::setShowCameraRotationCenter( bool enabled )
   emit showCameraRotationCenterChanged();
 }
 
+bool Qgs3DMapSettings::showCameraRotationCenter() const
+{
+  QGIS_PROTECT_QOBJECT_THREAD_ACCESS
+
+  return mShowCameraRotationCenter;
+}
 
 void Qgs3DMapSettings::setShowLightSourceOrigins( bool enabled )
 {
+  QGIS_PROTECT_QOBJECT_THREAD_ACCESS
+
   if ( mShowLightSources == enabled )
     return;
 
@@ -756,8 +951,17 @@ void Qgs3DMapSettings::setShowLightSourceOrigins( bool enabled )
   emit showLightSourceOriginsChanged();
 }
 
+bool Qgs3DMapSettings::showLightSourceOrigins() const
+{
+  QGIS_PROTECT_QOBJECT_THREAD_ACCESS
+
+  return mShowLightSources;
+}
+
 void Qgs3DMapSettings::setShowLabels( bool enabled )
 {
+  QGIS_PROTECT_QOBJECT_THREAD_ACCESS
+
   if ( mShowLabels == enabled )
     return;
 
@@ -765,37 +969,75 @@ void Qgs3DMapSettings::setShowLabels( bool enabled )
   emit showLabelsChanged();
 }
 
+bool Qgs3DMapSettings::showLabels() const
+{
+  QGIS_PROTECT_QOBJECT_THREAD_ACCESS
+
+  return mShowLabels;
+}
+
 void Qgs3DMapSettings::setEyeDomeLightingEnabled( bool enabled )
 {
+  QGIS_PROTECT_QOBJECT_THREAD_ACCESS
+
   if ( mEyeDomeLightingEnabled == enabled )
     return;
   mEyeDomeLightingEnabled = enabled;
   emit eyeDomeLightingEnabledChanged();
 }
 
+bool Qgs3DMapSettings::eyeDomeLightingEnabled() const
+{
+  QGIS_PROTECT_QOBJECT_THREAD_ACCESS
+
+  return mEyeDomeLightingEnabled;
+}
+
 void Qgs3DMapSettings::setEyeDomeLightingStrength( double strength )
 {
+  QGIS_PROTECT_QOBJECT_THREAD_ACCESS
+
   if ( mEyeDomeLightingStrength == strength )
     return;
   mEyeDomeLightingStrength = strength;
   emit eyeDomeLightingStrengthChanged();
 }
 
+double Qgs3DMapSettings::eyeDomeLightingStrength() const
+{
+  QGIS_PROTECT_QOBJECT_THREAD_ACCESS
+
+  return mEyeDomeLightingStrength;
+}
+
 void Qgs3DMapSettings::setEyeDomeLightingDistance( int distance )
 {
+  QGIS_PROTECT_QOBJECT_THREAD_ACCESS
+
   if ( mEyeDomeLightingDistance == distance )
     return;
   mEyeDomeLightingDistance = distance;
   emit eyeDomeLightingDistanceChanged();
 }
 
+int Qgs3DMapSettings::eyeDomeLightingDistance() const
+{
+  QGIS_PROTECT_QOBJECT_THREAD_ACCESS
+
+  return mEyeDomeLightingDistance;
+}
+
 QList<QgsLightSource *> Qgs3DMapSettings::lightSources() const
 {
+  QGIS_PROTECT_QOBJECT_THREAD_ACCESS
+
   return mLightSources;
 }
 
 void Qgs3DMapSettings::setLightSources( const QList<QgsLightSource *> &lights )
 {
+  QGIS_PROTECT_QOBJECT_THREAD_ACCESS
+
   // have lights actually changed?
   if ( mLightSources.count() == lights.count() )
   {
@@ -834,8 +1076,17 @@ void Qgs3DMapSettings::setLightSources( const QList<QgsLightSource *> &lights )
   emit lightSourcesChanged();
 }
 
+float Qgs3DMapSettings::fieldOfView() const
+{
+  QGIS_PROTECT_QOBJECT_THREAD_ACCESS
+
+  return mFieldOfView;
+}
+
 void Qgs3DMapSettings::setFieldOfView( const float fieldOfView )
 {
+  QGIS_PROTECT_QOBJECT_THREAD_ACCESS
+
   if ( mFieldOfView == fieldOfView )
     return;
 
@@ -843,8 +1094,17 @@ void Qgs3DMapSettings::setFieldOfView( const float fieldOfView )
   emit fieldOfViewChanged();
 }
 
+Qt3DRender::QCameraLens::ProjectionType Qgs3DMapSettings::projectionType() const
+{
+  QGIS_PROTECT_QOBJECT_THREAD_ACCESS
+
+  return mProjectionType;
+}
+
 void Qgs3DMapSettings::setProjectionType( const Qt3DRender::QCameraLens::ProjectionType projectionType )
 {
+  QGIS_PROTECT_QOBJECT_THREAD_ACCESS
+
   if ( mProjectionType == projectionType )
     return;
 
@@ -852,8 +1112,17 @@ void Qgs3DMapSettings::setProjectionType( const Qt3DRender::QCameraLens::Project
   emit projectionTypeChanged();
 }
 
+Qgis::NavigationMode Qgs3DMapSettings::cameraNavigationMode() const
+{
+  QGIS_PROTECT_QOBJECT_THREAD_ACCESS
+
+  return mCameraNavigationMode;
+}
+
 void Qgs3DMapSettings::setCameraNavigationMode( Qgis::NavigationMode navigationMode )
 {
+  QGIS_PROTECT_QOBJECT_THREAD_ACCESS
+
   if ( mCameraNavigationMode == navigationMode )
     return;
 
@@ -861,8 +1130,17 @@ void Qgs3DMapSettings::setCameraNavigationMode( Qgis::NavigationMode navigationM
   emit cameraNavigationModeChanged();
 }
 
+double Qgs3DMapSettings::cameraMovementSpeed() const
+{
+  QGIS_PROTECT_QOBJECT_THREAD_ACCESS
+
+  return mCameraMovementSpeed;
+}
+
 void Qgs3DMapSettings::setCameraMovementSpeed( double movementSpeed )
 {
+  QGIS_PROTECT_QOBJECT_THREAD_ACCESS
+
   if ( mCameraMovementSpeed == movementSpeed )
     return;
 
@@ -870,50 +1148,169 @@ void Qgs3DMapSettings::setCameraMovementSpeed( double movementSpeed )
   emit cameraMovementSpeedChanged();
 }
 
+void Qgs3DMapSettings::setOutputDpi( const double dpi )
+{
+  QGIS_PROTECT_QOBJECT_THREAD_ACCESS
+
+  mDpi = dpi;
+}
+
+double Qgs3DMapSettings::outputDpi() const
+{
+  QGIS_PROTECT_QOBJECT_THREAD_ACCESS
+
+  return mDpi;
+}
+
+QgsSkyboxSettings Qgs3DMapSettings::skyboxSettings() const
+{
+  QGIS_PROTECT_QOBJECT_THREAD_ACCESS
+
+  return mSkyboxSettings;
+}
+
+QgsShadowSettings Qgs3DMapSettings::shadowSettings() const
+{
+  QGIS_PROTECT_QOBJECT_THREAD_ACCESS
+
+  return mShadowSettings;
+}
+
+QgsAmbientOcclusionSettings Qgs3DMapSettings::ambientOcclusionSettings() const
+{
+  QGIS_PROTECT_QOBJECT_THREAD_ACCESS
+
+  return mAmbientOcclusionSettings;
+}
+
 void Qgs3DMapSettings::setSkyboxSettings( const QgsSkyboxSettings &skyboxSettings )
 {
+  QGIS_PROTECT_QOBJECT_THREAD_ACCESS
+
   mSkyboxSettings = skyboxSettings;
   emit skyboxSettingsChanged();
 }
 
 void Qgs3DMapSettings::setShadowSettings( const QgsShadowSettings &shadowSettings )
 {
+  QGIS_PROTECT_QOBJECT_THREAD_ACCESS
+
   mShadowSettings = shadowSettings;
   emit shadowSettingsChanged();
 }
 
 void Qgs3DMapSettings::setAmbientOcclusionSettings( const QgsAmbientOcclusionSettings &ambientOcclusionSettings )
 {
+  QGIS_PROTECT_QOBJECT_THREAD_ACCESS
+
   mAmbientOcclusionSettings = ambientOcclusionSettings;
   emit ambientOcclusionSettingsChanged();
 }
 
+bool Qgs3DMapSettings::isSkyboxEnabled() const
+{
+  QGIS_PROTECT_QOBJECT_THREAD_ACCESS
+
+  return mIsSkyboxEnabled;
+}
+
+void Qgs3DMapSettings::setIsSkyboxEnabled( bool enabled )
+{
+  QGIS_PROTECT_QOBJECT_THREAD_ACCESS
+
+  mIsSkyboxEnabled = enabled;
+}
+
+bool Qgs3DMapSettings::isFpsCounterEnabled() const
+{
+  QGIS_PROTECT_QOBJECT_THREAD_ACCESS
+
+  return mIsFpsCounterEnabled;
+}
+
 void Qgs3DMapSettings::setDebugShadowMapSettings( bool enabled, Qt::Corner corner, double size )
 {
+  QGIS_PROTECT_QOBJECT_THREAD_ACCESS
+
   mDebugShadowMapEnabled = enabled;
   mDebugShadowMapCorner = corner;
   mDebugShadowMapSize = size;
   emit debugShadowMapSettingsChanged();
 }
 
+bool Qgs3DMapSettings::debugShadowMapEnabled() const
+{
+  QGIS_PROTECT_QOBJECT_THREAD_ACCESS
+
+  return mDebugShadowMapEnabled;
+}
+
+Qt::Corner Qgs3DMapSettings::debugShadowMapCorner() const
+{
+  QGIS_PROTECT_QOBJECT_THREAD_ACCESS
+
+  return mDebugShadowMapCorner;
+}
+
+double Qgs3DMapSettings::debugShadowMapSize() const
+{
+  QGIS_PROTECT_QOBJECT_THREAD_ACCESS
+
+  return mDebugShadowMapSize;
+}
+
 void Qgs3DMapSettings::setDebugDepthMapSettings( bool enabled, Qt::Corner corner, double size )
 {
+  QGIS_PROTECT_QOBJECT_THREAD_ACCESS
+
   mDebugDepthMapEnabled = enabled;
   mDebugDepthMapCorner = corner;
   mDebugDepthMapSize = size;
   emit debugDepthMapSettingsChanged();
 }
 
+bool Qgs3DMapSettings::debugDepthMapEnabled() const
+{
+  QGIS_PROTECT_QOBJECT_THREAD_ACCESS
+
+  return mDebugDepthMapEnabled;
+}
+
+Qt::Corner Qgs3DMapSettings::debugDepthMapCorner() const
+{
+  QGIS_PROTECT_QOBJECT_THREAD_ACCESS
+
+  return mDebugDepthMapCorner;
+}
+
+double Qgs3DMapSettings::debugDepthMapSize() const
+{
+  QGIS_PROTECT_QOBJECT_THREAD_ACCESS
+
+  return mDebugDepthMapSize;
+}
+
 void Qgs3DMapSettings::setIsFpsCounterEnabled( bool fpsCounterEnabled )
 {
+  QGIS_PROTECT_QOBJECT_THREAD_ACCESS
+
   if ( fpsCounterEnabled == mIsFpsCounterEnabled )
     return;
   mIsFpsCounterEnabled = fpsCounterEnabled;
   emit fpsCounterEnabledChanged( mIsFpsCounterEnabled );
 }
 
+bool Qgs3DMapSettings::terrainRenderingEnabled() const
+{
+  QGIS_PROTECT_QOBJECT_THREAD_ACCESS
+
+  return mTerrainRenderingEnabled;
+}
+
 void Qgs3DMapSettings::setTerrainRenderingEnabled( bool terrainRenderingEnabled )
 {
+  QGIS_PROTECT_QOBJECT_THREAD_ACCESS
+
   if ( terrainRenderingEnabled == mTerrainRenderingEnabled )
     return;
   mTerrainRenderingEnabled = terrainRenderingEnabled;
@@ -922,21 +1319,43 @@ void Qgs3DMapSettings::setTerrainRenderingEnabled( bool terrainRenderingEnabled
 
 Qgis::RendererUsage Qgs3DMapSettings::rendererUsage() const
 {
+  QGIS_PROTECT_QOBJECT_THREAD_ACCESS
+
   return mRendererUsage;
 }
 
 void Qgs3DMapSettings::setRendererUsage( Qgis::RendererUsage rendererUsage )
 {
+  QGIS_PROTECT_QOBJECT_THREAD_ACCESS
+
   mRendererUsage = rendererUsage;
 }
 
+Qgis::ViewSyncModeFlags Qgs3DMapSettings::viewSyncMode() const
+{
+  QGIS_PROTECT_QOBJECT_THREAD_ACCESS
+
+  return mViewSyncMode;
+}
+
 void Qgs3DMapSettings::setViewSyncMode( Qgis::ViewSyncModeFlags mode )
 {
+  QGIS_PROTECT_QOBJECT_THREAD_ACCESS
+
   mViewSyncMode = mode;
 }
 
+bool Qgs3DMapSettings::viewFrustumVisualizationEnabled() const
+{
+  QGIS_PROTECT_QOBJECT_THREAD_ACCESS
+
+  return mVisualizeViewFrustum;
+}
+
 void Qgs3DMapSettings::setViewFrustumVisualizationEnabled( bool enabled )
 {
+  QGIS_PROTECT_QOBJECT_THREAD_ACCESS
+
   if ( mVisualizeViewFrustum != enabled )
   {
     mVisualizeViewFrustum = enabled;
@@ -944,8 +1363,17 @@ void Qgs3DMapSettings::setViewFrustumVisualizationEnabled( bool enabled )
   }
 }
 
+Qgs3DAxisSettings Qgs3DMapSettings::get3DAxisSettings() const
+{
+  QGIS_PROTECT_QOBJECT_THREAD_ACCESS
+
+  return m3dAxisSettings;
+}
+
 void Qgs3DMapSettings::setIsDebugOverlayEnabled( bool debugOverlayEnabled )
 {
+  QGIS_PROTECT_QOBJECT_THREAD_ACCESS
+
   if ( debugOverlayEnabled == mIsDebugOverlayEnabled )
     return;
 
@@ -953,9 +1381,17 @@ void Qgs3DMapSettings::setIsDebugOverlayEnabled( bool debugOverlayEnabled )
   emit debugOverlayEnabledChanged( mIsDebugOverlayEnabled );
 }
 
+bool Qgs3DMapSettings::showExtentIn2DView() const
+{
+  QGIS_PROTECT_QOBJECT_THREAD_ACCESS
+
+  return mShowExtentIn2DView;
+}
 
 void Qgs3DMapSettings::connectChangedSignalsToSettingsChanged()
 {
+  QGIS_PROTECT_QOBJECT_THREAD_ACCESS
+
   connect( this, &Qgs3DMapSettings::selectionColorChanged, this, &Qgs3DMapSettings::settingsChanged );
   connect( this, &Qgs3DMapSettings::layersChanged, this, &Qgs3DMapSettings::settingsChanged );
   connect( this, &Qgs3DMapSettings::terrainGeneratorChanged, this, &Qgs3DMapSettings::settingsChanged );
@@ -994,6 +1430,8 @@ void Qgs3DMapSettings::connectChangedSignalsToSettingsChanged()
 
 void Qgs3DMapSettings::set3DAxisSettings( const Qgs3DAxisSettings &axisSettings, bool force )
 {
+  QGIS_PROTECT_QOBJECT_THREAD_ACCESS
+
   if ( axisSettings == m3dAxisSettings )
   {
     if ( force )
@@ -1011,8 +1449,17 @@ void Qgs3DMapSettings::set3DAxisSettings( const Qgs3DAxisSettings &axisSettings,
   }
 }
 
+bool Qgs3DMapSettings::isDebugOverlayEnabled() const
+{
+  QGIS_PROTECT_QOBJECT_THREAD_ACCESS
+
+  return mIsDebugOverlayEnabled;
+}
+
 void Qgs3DMapSettings::setShowExtentIn2DView( bool show )
 {
+  QGIS_PROTECT_QOBJECT_THREAD_ACCESS
+
   if ( show == mShowExtentIn2DView )
     return;
 
