@@ -107,6 +107,11 @@ QString QgsExtractLabelsAlgorithm::shortDescription() const
   return QObject::tr( "Converts map labels to a point layer with relevant details saved as attributes." );
 }
 
+Qgis::ProcessingAlgorithmDocumentationFlags QgsExtractLabelsAlgorithm::documentationFlags() const
+{
+  return Qgis::ProcessingAlgorithmDocumentationFlag::RegeneratesPrimaryKey;
+}
+
 QString QgsExtractLabelsAlgorithm::shortHelpString() const
 {
   return QObject::tr( "This algorithm extracts label information from a rendered map at a given extent and scale.\n\n"
@@ -213,23 +218,23 @@ class ExtractLabelSink : public QgsLabelSink
           break;
 
         case Qgis::LabelMultiLineAlignment::FollowPlacement:
-          switch ( label->getQuadrant() )
+          switch ( label->quadrant() )
           {
-            case pal::LabelPosition::QuadrantAboveLeft:
-            case pal::LabelPosition::QuadrantLeft:
-            case pal::LabelPosition::QuadrantBelowLeft:
+            case Qgis::LabelQuadrantPosition::AboveLeft:
+            case Qgis::LabelQuadrantPosition::Left:
+            case Qgis::LabelQuadrantPosition::BelowLeft:
               labelAlignment = QStringLiteral( "right" );
               break;
 
-            case pal::LabelPosition::QuadrantAbove:
-            case pal::LabelPosition::QuadrantOver:
-            case pal::LabelPosition::QuadrantBelow:
+            case Qgis::LabelQuadrantPosition::Above:
+            case Qgis::LabelQuadrantPosition::Over:
+            case Qgis::LabelQuadrantPosition::Below:
               labelAlignment = QStringLiteral( "center" );
               break;
 
-            case pal::LabelPosition::QuadrantAboveRight:
-            case pal::LabelPosition::QuadrantRight:
-            case pal::LabelPosition::QuadrantBelowRight:
+            case Qgis::LabelQuadrantPosition::AboveRight:
+            case Qgis::LabelQuadrantPosition::Right:
+            case Qgis::LabelQuadrantPosition::BelowRight:
               labelAlignment = QStringLiteral( "left" );
               break;
           }
