@@ -22,6 +22,7 @@
 #include "qgslayoutmodel.h"
 #include "qgslayertree.h"
 #include "qgslayertreemodel.h"
+#include "qgslayertreefilterproxymodel.h"
 #include "qgslegendrenderer.h"
 #include "qgslegendstyle.h"
 #include "qgslogger.h"
@@ -105,6 +106,8 @@ void QgsLayoutItemLegend::paint( QPainter *painter, const QStyleOptionGraphicsIt
   if ( !painter )
     return;
 
+  const QPointF oldPos = pos();
+
   ensureModelIsInitialized();
 
   if ( mFilterAskedForUpdate )
@@ -113,14 +116,6 @@ void QgsLayoutItemLegend::paint( QPainter *painter, const QStyleOptionGraphicsIt
     doUpdateFilterByMap();
   }
 
-  if ( mLayout )
-  {
-    if ( !mLayout->renderContext().isPreviewRender() && mLegendModel->hitTestInProgress() )
-    {
-      mLegendModel->waitForHitTestBlocking();
-    }
-  }
-
   const int dpi = painter->device()->logicalDpiX();
   const double dotsPerMM = dpi / 25.4;
 
@@ -153,11 +148,9 @@ void QgsLayoutItemLegend::paint( QPainter *painter, const QStyleOptionGraphicsIt
   }
   mInitialMapScaleCalculated = true;
 
-  QgsLegendRenderer legendRenderer( mLegendModel.get(), mSettings );
+  QgsLegendRenderer legendRenderer = createRenderer();
   legendRenderer.setLegendSize( mForceResize && mSizeToContents ? QSize() : rect().size() );
 
-  const QPointF oldPos = pos();
-
   //adjust box if width or height is too small
   if ( mSizeToContents )
   {
@@ -265,7 +258,7 @@ void QgsLayoutItemLegend::draw( QgsLayoutItemRenderContext &context )
     Q_NOWARN_DEPRECATED_POP
   }
 
-  QgsLegendRenderer legendRenderer( mLegendModel.get(), mSettings );
+  QgsLegendRenderer legendRenderer = createRenderer();
   legendRenderer.setLegendSize( rect().size() );
 
   legendRenderer.drawLegend( rc );
@@ -288,7 +281,7 @@ void QgsLayoutItemLegend::adjustBoxSize()
   QgsRenderContext context = mMap ? QgsLayoutUtils::createRenderContextForMap( mMap, nullptr ) :
                              QgsLayoutUtils::createRenderContextForLayout( mLayout, nullptr );
 
-  QgsLegendRenderer legendRenderer( mLegendModel.get(), mSettings );
+  QgsLegendRenderer legendRenderer = createRenderer();
   const QSizeF size = legendRenderer.minimumSize( &context );
   QgsDebugMsgLevel( QStringLiteral( "width = %1 height = %2" ).arg( size.width() ).arg( size.height() ), 2 );
   if ( size.isValid() )
@@ -329,6 +322,16 @@ void QgsLayoutItemLegend::ensureModelIsInitialized() const
   }
 }
 
+QgsLegendRenderer QgsLayoutItemLegend::createRenderer() const
+{
+  QgsLegendRenderer res( mLegendModel.get(), mSettings );
+
+  // only show private layers when not in auto update mode
+  res.proxyModel()->setShowPrivateLayers( static_cast< bool >( mCustomLayerTree ) );
+
+  return res;
+}
+
 QgsLegendModel *QgsLayoutItemLegend::model()
 {
   ensureModelIsInitialized();
@@ -1130,6 +1133,10 @@ void QgsLayoutItemLegend::doUpdateFilterByMap()
     mLegendModel->setLayerStyleOverrides( QMap<QString, QString>() );
   }
 
+  // only use thread hit tests for preview renders. In other cases we'll need a blocking hit test anyway, and we run a risk
+  // of deadlocks if a non-preview render is then started on the main thread.
+  mLegendModel->setFlag( QgsLayerTreeModel::UseThreadedHitTest, mLayout->renderContext().isPreviewRender() );
+
   const bool filterByExpression = QgsLayerTreeUtils::hasLegendFilterExpression( *( mCustomLayerTree ? mCustomLayerTree.get() : mLayout->project()->layerTreeRoot() ) );
 
   const bool hasValidFilter = filterByExpression
@@ -1207,7 +1214,7 @@ void QgsLayoutItemLegend::doUpdateFilterByMap()
         {
           mapExtent.transform( mapTransform );
         }
-        catch ( QgsCsException &cse )
+        catch ( QgsCsException & )
         {
           continue;
         }
