@@ -38,7 +38,9 @@
 #include "lazperf/readers.hpp"
 
 #if defined(_MSC_VER)
+#ifndef UNICODE
 #define UNICODE
+#endif
 #include <locale>
 #include <codecvt>
 #endif
@@ -313,7 +315,9 @@ void decodePoint( char *buf, int lasPointFormat, char *dataBuffer, std::size_t &
   lazperf::las::nir14 nir;
   lazperf::las::point14 p14;
 
-  bool isLas14 = ( lasPointFormat == 6 || lasPointFormat == 7 || lasPointFormat == 8 );
+  // Does the point record start with the common fields for formats introduced
+  // in the LAS 1.4 spec?
+  const bool isLas14 = ( lasPointFormat == 6 || lasPointFormat == 7 || lasPointFormat == 8 || lasPointFormat == 9 || lasPointFormat == 10 );
 
   switch ( lasPointFormat )
   {
@@ -395,7 +399,12 @@ void decodePoint( char *buf, int lasPointFormat, char *dataBuffer, std::size_t &
         lazStoreToStream_<unsigned char>( dataBuffer, outputOffset, requestedAttribute.type, isLas14 ? p14.eofFlag() : p10.edge_of_flight_line );
         break;
       case QgsLazDecoder::LazAttribute::ScanAngleRank:
-        lazStoreToStream_<char>( dataBuffer, outputOffset, requestedAttribute.type, char( isLas14 ? p14.scanAngle() : p10.scan_angle_rank ) );
+        lazStoreToStream_<float>( dataBuffer, outputOffset, requestedAttribute.type,
+                                  isLas14
+                                  // Formats from LAS 1.4 spec store the angle in 0.006 degree increments
+                                  ? p14.scanAngle() * 0.006f
+                                  // Older formats store integer values
+                                  : p10.scan_angle_rank );
         break;
       case QgsLazDecoder::LazAttribute::UserData:
         lazStoreToStream_<unsigned char>( dataBuffer, outputOffset, requestedAttribute.type, isLas14 ? p14.userData() : p10.user_data );
