@@ -131,7 +131,7 @@ void QgsVectorLayerEditBuffer::updateChangedAttributes( QgsFeature &f )
 
 bool QgsVectorLayerEditBuffer::addFeature( QgsFeature &f )
 {
-  if ( !( L->dataProvider()->capabilities() & QgsVectorDataProvider::AddFeatures ) )
+  if ( !( L->dataProvider()->capabilities() & Qgis::VectorProviderCapability::AddFeatures ) )
   {
     return false;
   }
@@ -150,7 +150,7 @@ bool QgsVectorLayerEditBuffer::addFeature( QgsFeature &f )
 
 bool QgsVectorLayerEditBuffer::addFeatures( QgsFeatureList &features )
 {
-  if ( !( L->dataProvider()->capabilities() & QgsVectorDataProvider::AddFeatures ) )
+  if ( !( L->dataProvider()->capabilities() & Qgis::VectorProviderCapability::AddFeatures ) )
     return false;
 
   // we don't want to emit layerModified for every added feature, rather just once for the batch lot
@@ -181,7 +181,7 @@ bool QgsVectorLayerEditBuffer::addFeatures( QgsFeatureList &features )
 
 bool QgsVectorLayerEditBuffer::deleteFeature( QgsFeatureId fid )
 {
-  if ( !( L->dataProvider()->capabilities() & QgsVectorDataProvider::DeleteFeatures ) )
+  if ( !( L->dataProvider()->capabilities() & Qgis::VectorProviderCapability::DeleteFeatures ) )
   {
     QgsDebugError( QStringLiteral( "Cannot delete features (missing DeleteFeature capability)" ) );
     return false;
@@ -210,7 +210,7 @@ bool QgsVectorLayerEditBuffer::deleteFeature( QgsFeatureId fid )
 
 bool QgsVectorLayerEditBuffer::deleteFeatures( const QgsFeatureIds &fids )
 {
-  if ( !( L->dataProvider()->capabilities() & QgsVectorDataProvider::DeleteFeatures ) )
+  if ( !( L->dataProvider()->capabilities() & Qgis::VectorProviderCapability::DeleteFeatures ) )
   {
     QgsDebugError( QStringLiteral( "Cannot delete features (missing DeleteFeatures capability)" ) );
     return false;
@@ -243,7 +243,7 @@ bool QgsVectorLayerEditBuffer::changeGeometry( QgsFeatureId fid, const QgsGeomet
     if ( !mAddedFeatures.contains( fid ) )
       return false;
   }
-  else if ( !( L->dataProvider()->capabilities() & QgsVectorDataProvider::ChangeGeometries ) )
+  else if ( !( L->dataProvider()->capabilities() & Qgis::VectorProviderCapability::ChangeGeometries ) )
     return false;
 
   // TODO: check compatible geometry
@@ -285,7 +285,7 @@ bool QgsVectorLayerEditBuffer::changeAttributeValue( QgsFeatureId fid, int field
     if ( !mAddedFeatures.contains( fid ) )
       return false;
   }
-  else if ( !( L->dataProvider()->capabilities() & QgsVectorDataProvider::ChangeAttributeValues ) )
+  else if ( !( L->dataProvider()->capabilities() & Qgis::VectorProviderCapability::ChangeAttributeValues ) )
   {
     return false;
   }
@@ -302,7 +302,7 @@ bool QgsVectorLayerEditBuffer::changeAttributeValue( QgsFeatureId fid, int field
 
 bool QgsVectorLayerEditBuffer::addAttribute( const QgsField &field )
 {
-  if ( !( L->dataProvider()->capabilities() & QgsVectorDataProvider::AddAttributes ) )
+  if ( !( L->dataProvider()->capabilities() & Qgis::VectorProviderCapability::AddAttributes ) )
     return false;
 
   if ( field.name().isEmpty() )
@@ -325,7 +325,7 @@ bool QgsVectorLayerEditBuffer::addAttribute( const QgsField &field )
 
 bool QgsVectorLayerEditBuffer::deleteAttribute( int index )
 {
-  if ( !( L->dataProvider()->capabilities() & QgsVectorDataProvider::DeleteAttributes ) )
+  if ( !( L->dataProvider()->capabilities() & Qgis::VectorProviderCapability::DeleteAttributes ) )
     return false;
 
   if ( index < 0 || index >= L->fields().count() )
@@ -347,7 +347,7 @@ bool QgsVectorLayerEditBuffer::deleteAttribute( int index )
 
 bool QgsVectorLayerEditBuffer::renameAttribute( int index, const QString &newName )
 {
-  if ( !( L->dataProvider()->capabilities() & QgsVectorDataProvider::RenameAttributes ) )
+  if ( !( L->dataProvider()->capabilities() & Qgis::VectorProviderCapability::RenameAttributes ) )
     return false;
 
   if ( newName.isEmpty() )
@@ -605,7 +605,7 @@ bool QgsVectorLayerEditBuffer::commitChangesCheckGeometryTypeCompatibility( QStr
   if ( mAddedFeatures.isEmpty() )
     return true;
 
-  if ( L->dataProvider()->capabilities() & QgsVectorDataProvider::AddFeatures )
+  if ( L->dataProvider()->capabilities() & Qgis::VectorProviderCapability::AddFeatures )
   {
     if ( L->dataProvider()->doesStrictFeatureTypeCheck() )
     {
@@ -639,7 +639,7 @@ bool QgsVectorLayerEditBuffer::commitChangesDeleteAttributes( bool &attributesDe
   if ( mDeletedAttributeIds.isEmpty() )
     return true;
 
-  if ( ( L->dataProvider()->capabilities() & QgsVectorDataProvider::DeleteAttributes ) && L->dataProvider()->deleteAttributes( qgis::listToSet( mDeletedAttributeIds ) ) )
+  if ( ( L->dataProvider()->capabilities() & Qgis::VectorProviderCapability::DeleteAttributes ) && L->dataProvider()->deleteAttributes( qgis::listToSet( mDeletedAttributeIds ) ) )
   {
     commitErrors << tr( "SUCCESS: %n attribute(s) deleted.", "deleted attributes count", mDeletedAttributeIds.size() );
 
@@ -673,7 +673,7 @@ bool QgsVectorLayerEditBuffer::commitChangesRenameAttributes( bool &attributesRe
   if ( mRenamedAttributes.isEmpty() )
     return true;
 
-  if ( ( L->dataProvider()->capabilities() & QgsVectorDataProvider::RenameAttributes ) && L->dataProvider()->renameAttributes( mRenamedAttributes ) )
+  if ( ( L->dataProvider()->capabilities() & Qgis::VectorProviderCapability::RenameAttributes ) && L->dataProvider()->renameAttributes( mRenamedAttributes ) )
   {
     commitErrors << tr( "SUCCESS: %n attribute(s) renamed.", "renamed attributes count", mRenamedAttributes.size() );
 
@@ -698,7 +698,7 @@ bool QgsVectorLayerEditBuffer::commitChangesAddAttributes( bool &attributesAdded
   if ( mAddedAttributes.isEmpty() )
     return true;
 
-  if ( ( L->dataProvider()->capabilities()  & QgsVectorDataProvider::AddAttributes ) && L->dataProvider()->addAttributes( mAddedAttributes ) )
+  if ( ( L->dataProvider()->capabilities()  & Qgis::VectorProviderCapability::AddAttributes ) && L->dataProvider()->addAttributes( mAddedAttributes ) )
   {
     commitErrors << tr( "SUCCESS: %n attribute(s) added.", "added attributes count", mAddedAttributes.size() );
     emit committedAttributesAdded( L->id(), mAddedAttributes );
@@ -769,10 +769,10 @@ bool QgsVectorLayerEditBuffer::commitChangesChangeAttributes( bool &attributesCh
 {
   attributesChanged = false;
 
-  if ( L->dataProvider()->capabilities() & QgsVectorDataProvider::ChangeFeatures && !mChangedGeometries.isEmpty() && !mChangedAttributeValues.isEmpty() )
+  if ( L->dataProvider()->capabilities() & Qgis::VectorProviderCapability::ChangeFeatures && !mChangedGeometries.isEmpty() && !mChangedAttributeValues.isEmpty() )
   {
     // cppcheck-suppress assertWithSideEffect
-    Q_ASSERT( ( L->dataProvider()->capabilities() & ( QgsVectorDataProvider::ChangeAttributeValues | QgsVectorDataProvider::ChangeGeometries ) ) == ( QgsVectorDataProvider::ChangeAttributeValues | QgsVectorDataProvider::ChangeGeometries ) );
+    Q_ASSERT( ( L->dataProvider()->capabilities() & ( Qgis::VectorProviderCapability::ChangeAttributeValues | Qgis::VectorProviderCapability::ChangeGeometries ) ) == ( Qgis::VectorProviderCapability::ChangeAttributeValues | Qgis::VectorProviderCapability::ChangeGeometries ) );
 
     if ( L->dataProvider()->changeFeatures( mChangedAttributeValues, mChangedGeometries ) )
     {
@@ -793,8 +793,8 @@ bool QgsVectorLayerEditBuffer::commitChangesChangeAttributes( bool &attributesCh
 
   if ( !mChangedGeometries.isEmpty() )
   {
-    if ( ! L->dataProvider()->capabilities().testFlag( QgsVectorDataProvider::ChangeFeatures )
-         && ! L->dataProvider()->capabilities().testFlag( QgsVectorDataProvider::ChangeGeometries ) )
+    if ( ! L->dataProvider()->capabilities().testFlag( Qgis::VectorProviderCapability::ChangeFeatures )
+         && ! L->dataProvider()->capabilities().testFlag( Qgis::VectorProviderCapability::ChangeGeometries ) )
     {
       commitErrors << tr( "ERROR: %1 geometries not changed. Data provider '%2' does not have ChangeFeatures or ChangeGeometries capabilities", "not changed geometries count" )
                    .arg( mChangedGeometries.size() )
@@ -818,8 +818,8 @@ bool QgsVectorLayerEditBuffer::commitChangesChangeAttributes( bool &attributesCh
 
   if ( !mChangedAttributeValues.isEmpty() )
   {
-    if ( ! L->dataProvider()->capabilities().testFlag( QgsVectorDataProvider::ChangeFeatures )
-         && ! L->dataProvider()->capabilities().testFlag( QgsVectorDataProvider::ChangeAttributeValues ) )
+    if ( ! L->dataProvider()->capabilities().testFlag( Qgis::VectorProviderCapability::ChangeFeatures )
+         && ! L->dataProvider()->capabilities().testFlag( Qgis::VectorProviderCapability::ChangeAttributeValues ) )
     {
       commitErrors << tr( "ERROR: %1 attribute value change(s) not applied. Data provider '%2' does not have ChangeFeatures or ChangeAttributeValues capabilities", "not changed attribute values count" )
                    .arg( mChangedAttributeValues.size() )
@@ -866,7 +866,7 @@ bool QgsVectorLayerEditBuffer::commitChangesDeleteFeatures( bool &featuresDelete
   if ( mDeletedFeatureIds.isEmpty() )
     return true;
 
-  if ( ( L->dataProvider()->capabilities() & QgsVectorDataProvider::DeleteFeatures ) && L->dataProvider()->deleteFeatures( mDeletedFeatureIds ) )
+  if ( ( L->dataProvider()->capabilities() & Qgis::VectorProviderCapability::DeleteFeatures ) && L->dataProvider()->deleteFeatures( mDeletedFeatureIds ) )
   {
     commitErrors << tr( "SUCCESS: %n feature(s) deleted.", "deleted features count", mDeletedFeatureIds.size() );
     featuresDeleted = true;
@@ -906,7 +906,7 @@ bool QgsVectorLayerEditBuffer::commitChangesAddFeatures( bool &featuresAdded, QS
   if ( mAddedFeatures.isEmpty() )
     return true;
 
-  if ( L->dataProvider()->capabilities() & QgsVectorDataProvider::AddFeatures )
+  if ( L->dataProvider()->capabilities() & Qgis::VectorProviderCapability::AddFeatures )
   {
     QList<QgsFeatureId> ids;
     QgsFeatureList featuresToAdd;
