@@ -113,6 +113,45 @@ class QgsElevationProfilePlotItem : public Qgs2DPlot, public QgsPlotCanvasItem
         case Qgis::DistanceUnit::Centimeters:
         case Qgis::DistanceUnit::Millimeters:
         case Qgis::DistanceUnit::Inches:
+        case Qgis::DistanceUnit::ChainsInternational:
+        case Qgis::DistanceUnit::ChainsBritishBenoit1895A:
+        case Qgis::DistanceUnit::ChainsBritishBenoit1895B:
+        case Qgis::DistanceUnit::ChainsBritishSears1922Truncated:
+        case Qgis::DistanceUnit::ChainsBritishSears1922:
+        case Qgis::DistanceUnit::ChainsClarkes:
+        case Qgis::DistanceUnit::ChainsUSSurvey:
+        case Qgis::DistanceUnit::FeetBritish1865:
+        case Qgis::DistanceUnit::FeetBritish1936:
+        case Qgis::DistanceUnit::FeetBritishBenoit1895A:
+        case Qgis::DistanceUnit::FeetBritishBenoit1895B:
+        case Qgis::DistanceUnit::FeetBritishSears1922Truncated:
+        case Qgis::DistanceUnit::FeetBritishSears1922:
+        case Qgis::DistanceUnit::FeetClarkes:
+        case Qgis::DistanceUnit::FeetGoldCoast:
+        case Qgis::DistanceUnit::FeetIndian:
+        case Qgis::DistanceUnit::FeetIndian1937:
+        case Qgis::DistanceUnit::FeetIndian1962:
+        case Qgis::DistanceUnit::FeetIndian1975:
+        case Qgis::DistanceUnit::FeetUSSurvey:
+        case Qgis::DistanceUnit::LinksInternational:
+        case Qgis::DistanceUnit::LinksBritishBenoit1895A:
+        case Qgis::DistanceUnit::LinksBritishBenoit1895B:
+        case Qgis::DistanceUnit::LinksBritishSears1922Truncated:
+        case Qgis::DistanceUnit::LinksBritishSears1922:
+        case Qgis::DistanceUnit::LinksClarkes:
+        case Qgis::DistanceUnit::LinksUSSurvey:
+        case Qgis::DistanceUnit::YardsBritishBenoit1895A:
+        case Qgis::DistanceUnit::YardsBritishBenoit1895B:
+        case Qgis::DistanceUnit::YardsBritishSears1922Truncated:
+        case Qgis::DistanceUnit::YardsBritishSears1922:
+        case Qgis::DistanceUnit::YardsClarkes:
+        case Qgis::DistanceUnit::YardsIndian:
+        case Qgis::DistanceUnit::YardsIndian1937:
+        case Qgis::DistanceUnit::YardsIndian1962:
+        case Qgis::DistanceUnit::YardsIndian1975:
+        case Qgis::DistanceUnit::MilesUSSurvey:
+        case Qgis::DistanceUnit::Fathoms:
+        case Qgis::DistanceUnit::MetersGermanLegal:
           return QStringLiteral( " %1" ).arg( QgsUnitTypes::toAbbreviatedString( mDistanceUnit ) );
 
         case Qgis::DistanceUnit::Degrees:
@@ -825,13 +864,7 @@ void QgsElevationProfileCanvas::refresh()
   if ( !mProject || !profileCurve() )
     return;
 
-  if ( mCurrentJob )
-  {
-    mPlotItem->setRenderer( nullptr );
-    disconnect( mCurrentJob, &QgsProfilePlotRenderer::generationFinished, this, &QgsElevationProfileCanvas::generationFinished );
-    mCurrentJob->deleteLater();
-    mCurrentJob = nullptr;
-  }
+  cancelJobs();
 
   QgsProfileRequest request( profileCurve()->clone() );
   request.setCrs( mCrs );
@@ -1395,7 +1428,7 @@ QVector<QgsProfileIdentifyResults> QgsElevationProfileCanvas::identify( const QR
 void QgsElevationProfileCanvas::clear()
 {
   setProfileCurve( nullptr );
-  mPlotItem->setRenderer( nullptr );
+  cancelJobs();
   mPlotItem->updatePlot();
 }
 
