@@ -38,7 +38,8 @@
 #include "qgsprocessinghelpeditorwidget.h"
 #include "qgsscreenhelper.h"
 #include "qgsmessagelog.h"
-
+#include "qgsprocessingalgorithmdialogbase.h"
+#include "qgsproject.h"
 #include <QShortcut>
 #include <QKeySequence>
 #include <QFileDialog>
@@ -86,6 +87,8 @@ QgsModelDesignerDialog::QgsModelDesignerDialog( QWidget *parent, Qt::WindowFlags
 {
   setupUi( this );
 
+  mLayerStore.setProject( QgsProject::instance() );
+
   mScreenHelper = new QgsScreenHelper( this );
 
   setAttribute( Qt::WA_DeleteOnClose );
@@ -157,6 +160,8 @@ QgsModelDesignerDialog::QgsModelDesignerDialog( QWidget *parent, Qt::WindowFlags
   connect( mActionReorderOutputs, &QAction::triggered, this, &QgsModelDesignerDialog::reorderOutputs );
   connect( mActionEditHelp, &QAction::triggered, this, &QgsModelDesignerDialog::editHelp );
   connect( mReorderInputsButton, &QPushButton::clicked, this, &QgsModelDesignerDialog::reorderInputs );
+  connect( mActionRun, &QAction::triggered, this, [this] { run(); } );
+  connect( mActionRunSelectedSteps, &QAction::triggered, this, &QgsModelDesignerDialog::runSelectedSteps );
 
   mActionSnappingEnabled->setChecked( settings.value( QStringLiteral( "/Processing/Modeler/enableSnapToGrid" ), false ).toBool() );
   connect( mActionSnappingEnabled, &QAction::toggled, this, [ = ]( bool enabled )
@@ -306,6 +311,7 @@ QgsModelDesignerDialog::QgsModelDesignerDialog( QWidget *parent, Qt::WindowFlags
       beginUndoCommand( tr( "Change Model Group" ), GroupChanged );
       mModel->setGroup( group );
       endUndoCommand();
+      updateWindowTitle();
     }
   } );
 
@@ -489,7 +495,7 @@ void QgsModelDesignerDialog::setModelScene( QgsModelGraphicsScene *scene )
 
   mScene = scene;
   mScene->setParent( this );
-  mScene->setChildAlgorithmResults( mChildResults );
+  mScene->setLastRunResult( mLastResult );
   mScene->setModel( mModel.get() );
   mScene->setMessageBar( mMessageBar );
 
@@ -508,6 +514,10 @@ void QgsModelDesignerDialog::setModelScene( QgsModelGraphicsScene *scene )
   } );
   connect( mScene, &QgsModelGraphicsScene::componentAboutToChange, this, [ = ]( const QString & description, int id ) { beginUndoCommand( description, id ); } );
   connect( mScene, &QgsModelGraphicsScene::componentChanged, this, [ = ] { endUndoCommand(); } );
+  connect( mScene, &QgsModelGraphicsScene::runFromChild, this, &QgsModelDesignerDialog::runFromChild );
+  connect( mScene, &QgsModelGraphicsScene::runSelected, this, &QgsModelDesignerDialog::runSelectedSteps );
+  connect( mScene, &QgsModelGraphicsScene::showChildAlgorithmOutputs, this, &QgsModelDesignerDialog::showChildAlgorithmOutputs );
+  connect( mScene, &QgsModelGraphicsScene::showChildAlgorithmLog, this, &QgsModelDesignerDialog::showChildAlgorithmLog );
 
   mView->centerOn( center );
 
@@ -590,18 +600,11 @@ bool QgsModelDesignerDialog::checkForUnsavedChanges()
   }
 }
 
-void QgsModelDesignerDialog::setLastRunChildAlgorithmResults( const QVariantMap &results )
+void QgsModelDesignerDialog::setLastRunResult( const QgsProcessingModelResult &result )
 {
-  mChildResults = results;
+  mLastResult.mergeWith( result );
   if ( mScene )
-    mScene->setChildAlgorithmResults( mChildResults );
-}
-
-void QgsModelDesignerDialog::setLastRunChildAlgorithmInputs( const QVariantMap &inputs )
-{
-  mChildInputs = inputs;
-  if ( mScene )
-    mScene->setChildAlgorithmInputs( mChildInputs );
+    mScene->setLastRunResult( mLastResult );
 }
 
 void QgsModelDesignerDialog::setModelName( const QString &name )
@@ -821,7 +824,9 @@ void QgsModelDesignerDialog::updateWindowTitle()
 {
   QString title = tr( "Model Designer" );
   if ( !mModel->name().isEmpty() )
-    title = QStringLiteral( "%1 - %2" ).arg( title, mModel->name() );
+    title = mModel->group().isEmpty()
+            ? QStringLiteral( "%1: %2" ).arg( title, mModel->name() )
+            : QStringLiteral( "%1: %2 - %3" ).arg( title, mModel->group(), mModel->name() );
 
   if ( isDirty() )
     title.prepend( '*' );
@@ -994,6 +999,229 @@ void QgsModelDesignerDialog::editHelp()
   }
 }
 
+void QgsModelDesignerDialog::runSelectedSteps()
+{
+  QSet<QString> children;
+  const QList< QgsModelComponentGraphicItem * > items = mScene->selectedComponentItems();
+  for ( QgsModelComponentGraphicItem *item : items )
+  {
+    if ( QgsProcessingModelChildAlgorithm *childAlgorithm = dynamic_cast< QgsProcessingModelChildAlgorithm *>( item->component() ) )
+    {
+      children.insert( childAlgorithm->childId() );
+    }
+  }
+
+  if ( children.isEmpty() )
+  {
+    mMessageBar->pushWarning( QString(), tr( "No steps are selected" ) );
+    return;
+  }
+
+  run( children );
+}
+
+void QgsModelDesignerDialog::runFromChild( const QString &id )
+{
+  QSet<QString> children = mModel->dependentChildAlgorithms( id );
+  children.insert( id );
+  run( children );
+}
+
+void QgsModelDesignerDialog::run( const QSet<QString> &childAlgorithmSubset )
+{
+  QStringList errors;
+  const bool isValid = model()->validate( errors );
+  if ( !isValid )
+  {
+    QMessageBox messageBox;
+    messageBox.setWindowTitle( tr( "Model is Invalid" ) );
+    messageBox.setIcon( QMessageBox::Icon::Warning );
+    messageBox.setText( tr( "This model is not valid and contains one or more issues. Are you sure you want to run it in this state?" ) );
+    messageBox.setStandardButtons( QMessageBox::StandardButton::Yes | QMessageBox::StandardButton::Cancel );
+    messageBox.setDefaultButton( QMessageBox::StandardButton::Cancel );
+
+    QString errorString;
+    for ( const QString &error : std::as_const( errors ) )
+    {
+      QString cleanedError = error;
+      const thread_local QRegularExpression re( QStringLiteral( "<[^>]*>" ) );
+      cleanedError.replace( re, QString() );
+      errorString += QStringLiteral( "• %1\n" ).arg( cleanedError );
+    }
+
+    messageBox.setDetailedText( errorString );
+    if ( messageBox.exec() == QMessageBox::StandardButton::Cancel )
+      return;
+  }
+
+  if ( !childAlgorithmSubset.isEmpty() )
+  {
+    for ( const QString &child : childAlgorithmSubset )
+    {
+      // has user previously run all requirements for this step?
+      const QSet< QString > requirements = mModel->dependsOnChildAlgorithms( child );
+      for ( const QString &requirement : requirements )
+      {
+        if ( !mLastResult.executedChildIds().contains( requirement ) )
+        {
+          QMessageBox messageBox;
+          messageBox.setWindowTitle( tr( "Run Model" ) );
+          messageBox.setIcon( QMessageBox::Icon::Warning );
+          messageBox.setText( tr( "Prerequisite parts of this model have not yet been run (try running the full model first)." ) );
+          messageBox.setStandardButtons( QMessageBox::StandardButton::Ok );
+          messageBox.exec();
+          return;
+        }
+      }
+    }
+  }
+
+  std::unique_ptr< QgsProcessingAlgorithmDialogBase > dialog( createExecutionDialog() );
+  if ( !dialog )
+    return;
+
+  dialog->setLogLevel( Qgis::ProcessingLogLevel::ModelDebug );
+  dialog->setParameters( mModel->designerParameterValues() );
+
+  connect( dialog.get(), &QgsProcessingAlgorithmDialogBase::algorithmAboutToRun, this, [this, &childAlgorithmSubset]( QgsProcessingContext * context )
+  {
+    if ( ! childAlgorithmSubset.empty() )
+    {
+      // start from previous state
+      std::unique_ptr< QgsProcessingModelInitialRunConfig > modelConfig = std::make_unique< QgsProcessingModelInitialRunConfig >();
+      modelConfig->setChildAlgorithmSubset( childAlgorithmSubset );
+      modelConfig->setPreviouslyExecutedChildAlgorithms( mLastResult.executedChildIds() );
+      modelConfig->setInitialChildInputs( mLastResult.rawChildInputs() );
+      modelConfig->setInitialChildOutputs( mLastResult.rawChildOutputs() );
+
+      // add copies of layers from previous runs to context's layer store, so that they can be used
+      // when running the subset
+      const QMap<QString, QgsMapLayer *> previousOutputLayers = mLayerStore.temporaryLayerStore()->mapLayers();
+      std::unique_ptr<QgsMapLayerStore> previousResultStore = std::make_unique< QgsMapLayerStore >();
+      for ( auto it = previousOutputLayers.constBegin(); it != previousOutputLayers.constEnd(); ++it )
+      {
+        std::unique_ptr< QgsMapLayer > clone( it.value()->clone() );
+        clone->setId( it.value()->id() );
+        previousResultStore->addMapLayer( clone.release() );
+      }
+      previousResultStore->moveToThread( nullptr );
+      modelConfig->setPreviousLayerStore( std::move( previousResultStore ) );
+      context->setModelInitialRunConfig( std::move( modelConfig ) );
+    }
+  } );
+
+  connect( dialog.get(), &QgsProcessingAlgorithmDialogBase::algorithmFinished, this, [this, &dialog]( bool, const QVariantMap & )
+  {
+    QgsProcessingContext *context = dialog->processingContext();
+
+    setLastRunResult( context->modelResult() );
+
+    mModel->setDesignerParameterValues( dialog->createProcessingParameters( QgsProcessingParametersGenerator::Flag::SkipDefaultValueParameters ) );
+
+    // take child output layers
+    mLayerStore.temporaryLayerStore()->removeAllMapLayers();
+    mLayerStore.takeResultsFrom( *context );
+  } );
+
+  dialog->exec();
+}
+
+void QgsModelDesignerDialog::showChildAlgorithmOutputs( const QString &childId )
+{
+  const QString childDescription = mModel->childAlgorithm( childId ).description();
+
+  const QgsProcessingModelChildAlgorithmResult result = mLastResult.childResults().value( childId );
+  const QVariantMap childAlgorithmOutputs = result.outputs();
+  if ( childAlgorithmOutputs.isEmpty() )
+  {
+    mMessageBar->pushWarning( QString(), tr( "No results are available for %1" ).arg( childDescription ) );
+    return;
+  }
+
+  const QgsProcessingAlgorithm *algorithm = mModel->childAlgorithm( childId ).algorithm();
+  if ( !algorithm )
+  {
+    mMessageBar->pushCritical( QString(), tr( "Results cannot be shown for an invalid model component" ) );
+    return;
+  }
+
+  const QList< const QgsProcessingParameterDefinition * > outputParams = algorithm->destinationParameterDefinitions();
+  if ( outputParams.isEmpty() )
+  {
+    // this situation should not arise in normal use, we don't show the action in this case
+    QgsDebugError( "Cannot show results for algorithms with no outputs" );
+    return;
+  }
+
+  bool foundResults = false;
+  for ( const QgsProcessingParameterDefinition *outputParam : outputParams )
+  {
+    const QVariant output = childAlgorithmOutputs.value( outputParam->name() );
+    if ( !output.isValid() )
+      continue;
+
+    if ( output.type() == QVariant::String )
+    {
+      if ( QgsMapLayer *resultLayer = QgsProcessingUtils::mapLayerFromString( output.toString(), mLayerStore ) )
+      {
+        QgsDebugMsgLevel( QStringLiteral( "Loading previous result for %1: %2" ).arg( outputParam->name(), output.toString() ), 2 );
+
+        std::unique_ptr< QgsMapLayer > layer( resultLayer->clone() );
+
+        QString baseName;
+        if ( outputParams.size() > 1 )
+          baseName = tr( "%1 — %2" ).arg( childDescription, outputParam->name() );
+        else
+          baseName = childDescription;
+
+        // make name unique, so that's it's easy to see which is the most recent result.
+        // (this helps when running the model multiple times.)
+        QString name = baseName;
+        int counter = 1;
+        while ( !QgsProject::instance()->mapLayersByName( name ).empty() )
+        {
+          counter += 1;
+          name = tr( "%1 (%2)" ).arg( baseName ).arg( counter );
+        }
+
+        layer->setName( name );
+
+        QgsProject::instance()->addMapLayer( layer.release() );
+        foundResults = true;
+      }
+      else
+      {
+        // should not happen in normal operation
+        QgsDebugError( QStringLiteral( "Could not load previous result for %1: %2" ).arg( outputParam->name(), output.toString() ) );
+      }
+    }
+  }
+
+  if ( !foundResults )
+  {
+    mMessageBar->pushWarning( QString(), tr( "No results are available for %1" ).arg( childDescription ) );
+    return;
+  }
+}
+
+void QgsModelDesignerDialog::showChildAlgorithmLog( const QString &childId )
+{
+  const QString childDescription = mModel->childAlgorithm( childId ).description();
+
+  const QgsProcessingModelChildAlgorithmResult result = mLastResult.childResults().value( childId );
+  if ( result.htmlLog().isEmpty() )
+  {
+    mMessageBar->pushWarning( QString(), tr( "No log is available for %1" ).arg( childDescription ) );
+    return;
+  }
+
+  QgsMessageViewer m( this, QgsGuiUtils::ModalDialogFlags, false );
+  m.setWindowTitle( childDescription );
+  m.setCheckBoxVisible( false );
+  m.setMessageAsHtml( result.htmlLog() );
+  m.exec();
+}
+
 void QgsModelDesignerDialog::validate()
 {
   QStringList issues;
