@@ -33,6 +33,10 @@ void QgsSensorThingsDataItemGuiProvider::populateContextMenu( QgsDataItem *item,
     connect( actionEdit, &QAction::triggered, this, [connectionItem] { editConnection( connectionItem ); } );
     menu->addAction( actionEdit );
 
+    QAction *actionDuplicate = new QAction( tr( "Duplicate Connection" ), menu );
+    connect( actionDuplicate, &QAction::triggered, this, [connectionItem] { duplicateConnection( connectionItem ); } );
+    menu->addAction( actionDuplicate );
+
     const QList< QgsSensorThingsConnectionItem * > stConnectionItems = QgsDataItem::filteredItems<QgsSensorThingsConnectionItem>( selection );
     QAction *actionDelete = new QAction( stConnectionItems.size() > 1 ? tr( "Remove Connections…" ) : tr( "Remove Connection…" ), menu );
     connect( actionDelete, &QAction::triggered, this, [stConnectionItems, context]
@@ -83,6 +87,19 @@ void QgsSensorThingsDataItemGuiProvider::editConnection( QgsDataItem *item )
   item->parent()->refreshConnections();
 }
 
+void QgsSensorThingsDataItemGuiProvider::duplicateConnection( QgsDataItem *item )
+{
+  const QString connectionName = item->name();
+  const QgsSensorThingsProviderConnection::Data connection = QgsSensorThingsProviderConnection::connection( connectionName );
+  const QStringList connections = QgsSensorThingsProviderConnection::sTreeSensorThingsConnections->items();
+
+  const QString newConnectionName = QgsDataItemGuiProviderUtils::uniqueName( connectionName, connections );
+
+  QgsSensorThingsProviderConnection::addConnection( newConnectionName, connection );
+  item->parent()->refreshConnections();
+}
+
+
 void QgsSensorThingsDataItemGuiProvider::newConnection( QgsDataItem *item )
 {
   QgsSensorThingsConnectionDialog dlg;
