@@ -18,6 +18,7 @@
 #include "qgsdatasourceuri.h"
 #include "qgshanaconnection.h"
 #include "qgshanaconnectionstringbuilder.h"
+#include "qgshanadatatypes.h"
 #include "qgshanadriver.h"
 #include "qgshanaexception.h"
 #include "qgshanaresultset.h"
@@ -99,63 +100,62 @@ QgsField AttributeField::toQgsField() const
   QMetaType::Type fieldType;
   switch ( type )
   {
-    case SQLDataTypes::Bit:
-    case SQLDataTypes::Boolean:
+    case QgsHanaDataType::Bit:
+    case QgsHanaDataType::Boolean:
       fieldType = QMetaType::Type::Bool;
       break;
-    case SQLDataTypes::TinyInt:
-    case SQLDataTypes::SmallInt:
-    case SQLDataTypes::Integer:
+    case QgsHanaDataType::TinyInt:
+    case QgsHanaDataType::SmallInt:
+    case QgsHanaDataType::Integer:
       fieldType = isSigned ? QMetaType::Type::Int : QMetaType::Type::UInt;
       break;
-    case SQLDataTypes::BigInt:
+    case QgsHanaDataType::BigInt:
       fieldType = isSigned ? QMetaType::Type::LongLong : QMetaType::Type::ULongLong;
       break;
-    case SQLDataTypes::Numeric:
-    case SQLDataTypes::Decimal:
+    case QgsHanaDataType::Numeric:
+    case QgsHanaDataType::Decimal:
+    case QgsHanaDataType::Double:
+    case QgsHanaDataType::Float:
+    case QgsHanaDataType::Real:
       fieldType = QMetaType::Type::Double;
       break;
-    case SQLDataTypes::Double:
-    case SQLDataTypes::Float:
-    case SQLDataTypes::Real:
-      fieldType = QMetaType::Type::Double;
-      break;
-    case SQLDataTypes::Char:
-    case SQLDataTypes::WChar:
+    case QgsHanaDataType::Char:
+    case QgsHanaDataType::WChar:
       fieldType = ( size == 1 ) ? QMetaType::Type::QChar : QMetaType::Type::QString;
       break;
-    case SQLDataTypes::VarChar:
-    case SQLDataTypes::WVarChar:
-    case SQLDataTypes::LongVarChar:
-    case SQLDataTypes::WLongVarChar:
+    case QgsHanaDataType::VarChar:
+    case QgsHanaDataType::WVarChar:
+    case QgsHanaDataType::LongVarChar:
+    case QgsHanaDataType::WLongVarChar:
+    // There are two options how to treat ST_GEOMETRY columns that are attributes:
+    // 1. Type is QMetaType::Type::QString. The value is provided as WKT and editable.
+    // 2. Type is QMetaType::Type::QByteArray. The value is in WKB format and uneditable.
+    case QgsHanaDataType::Geometry:
+    // There are two options how to treat REAL_VECTOR columns that are attributes:
+    // 1. Type is QMetaType::Type::QString. The value has string representation in the format '[1.0,3.2,0.6]'.
+    // 2. Type is QMetaType::Type::QByteArray. The value has fvecs representation and uneditable.
+    case QgsHanaDataType::RealVector:
       fieldType = QMetaType::Type::QString;
       break;
-    case SQLDataTypes::Binary:
-    case SQLDataTypes::VarBinary:
-    case SQLDataTypes::LongVarBinary:
+    case QgsHanaDataType::Binary:
+    case QgsHanaDataType::VarBinary:
+    case QgsHanaDataType::LongVarBinary:
       fieldType = QMetaType::Type::QByteArray;
       break;
-    case SQLDataTypes::Date:
-    case SQLDataTypes::TypeDate:
+    case QgsHanaDataType::Date:
+    case QgsHanaDataType::TypeDate:
       fieldType = QMetaType::Type::QDate;
       break;
-    case SQLDataTypes::Time:
-    case SQLDataTypes::TypeTime:
+    case QgsHanaDataType::Time:
+    case QgsHanaDataType::TypeTime:
       fieldType = QMetaType::Type::QTime;
       break;
-    case SQLDataTypes::Timestamp:
-    case SQLDataTypes::TypeTimestamp:
+    case QgsHanaDataType::Timestamp:
+    case QgsHanaDataType::TypeTimestamp:
       fieldType = QMetaType::Type::QDateTime;
       break;
     default:
-      if ( isGeometry() )
-        // There are two options how to treat geometry columns that are attributes:
-        // 1. Type is QVariant::String. The value is provided as WKT and editable.
-        // 2. Type is QVariant::ByteArray. The value is provided as BLOB and uneditable.
-        fieldType = QMetaType::Type::QString;
-      else
-        throw QgsHanaException( QString( "Field type '%1' is not supported" ).arg( QString::number( type ) ) );
-      break;
+      throw QgsHanaException( QString( "Field type '%1' is not supported" ).arg( QString::number( static_cast<int>( type ) ) ) );
   }
 
   QgsField field = QgsField( name, fieldType, typeName, size, precision, comment, QMetaType::Type::UnknownType );
@@ -168,6 +168,10 @@ QgsField AttributeField::toQgsField() const
       constraints.setConstraint( QgsFieldConstraints::ConstraintUnique, QgsFieldConstraints::ConstraintOriginProvider );
     field.setConstraints( constraints );
   }
+
+  if ( type == QgsHanaDataType::Geometry )
+    field.setMetadata( Qgis::FieldMetadataProperty::CustomProperty, srid );
+
   return field;
 }
 
@@ -509,6 +513,14 @@ const QString &QgsHanaConnection::getDatabaseVersion()
   return mDatabaseVersion;
 }
 
+const QString &QgsHanaConnection::getDatabaseCloudVersion()
+{
+  if ( mDatabaseCloudVersion.isEmpty() && QgsHanaUtils::toHANAVersion( getDatabaseVersion() ).majorVersion() >= 4 )
+    mDatabaseCloudVersion = executeScalar( QStringLiteral( "SELECT CLOUD_VERSION FROM SYS.M_DATABASE" ) ).toString();
+
+  return mDatabaseCloudVersion;
+}
+
 const QString &QgsHanaConnection::getUserName()
 {
   if ( mUserName.isEmpty() )
@@ -758,13 +770,16 @@ void QgsHanaConnection::readQueryFields( const QString &schemaName, const QStrin
       field.tableName = QString::fromStdU16String( rsmd->getTableName( i ) );
       field.name = QString::fromStdU16String( rsmd->getColumnName( i ) );
       field.typeName = QString::fromStdU16String( rsmd->getColumnTypeName( i ) );
-      field.type = rsmd->getColumnType( i );
+      field.type = QgsHanaDataTypeUtils::fromInt( rsmd->getColumnType( i ) );
       field.isSigned = rsmd->isSigned( i );
       field.isNullable = rsmd->isNullable( i );
       field.isAutoIncrement = rsmd->isAutoIncrement( i );
       field.size = static_cast<int>( rsmd->getColumnLength( i ) );
       field.precision = static_cast<int>( rsmd->getScale( i ) );
 
+      if ( field.type == QgsHanaDataType::Unknown )
+        throw QgsHanaException( QString( "Type of the column '%1' is unknown" ).arg( field.name ) );
+
       if ( !schema.isEmpty() )
       {
         field.isUnique = isColumnUnique( schema, baseTableName, baseColumnName );
@@ -822,21 +837,21 @@ void QgsHanaConnection::readTableFields( const QString &schemaName, const QStrin
       field.schemaName = rsColumns->getString( 2/*TABLE_SCHEM*/ );
       field.tableName = rsColumns->getString( 3/*TABLE_NAME*/ );
       field.name = rsColumns->getString( 4/*COLUMN_NAME*/ );
-      field.type = rsColumns->getShort( 5/*DATA_TYPE*/ );
+      field.type = QgsHanaDataTypeUtils::fromInt( rsColumns->getShort( 5/*DATA_TYPE*/ ) );
       field.typeName =  rsColumns->getString( 6/*TYPE_NAME*/ );
-      if ( field.type == SQLDataTypes::Unknown )
+      if ( field.type == QgsHanaDataType::Unknown )
         throw QgsHanaException( QString( "Type of the column '%1' is unknown" ).arg( field.name ) );
       field.size = rsColumns->getInt( 7/*COLUMN_SIZE*/ );
       field.precision = static_cast<int>( rsColumns->getShort( 9/*DECIMAL_DIGITS*/ ) );
-      field.isSigned = field.type == SQLDataTypes::SmallInt || field.type == SQLDataTypes::Integer ||
-                       field.type == SQLDataTypes::BigInt || field.type == SQLDataTypes::Decimal ||
-                       field.type == SQLDataTypes::Numeric || field.type == SQLDataTypes::Real ||
-                       field.type == SQLDataTypes::Float || field.type == SQLDataTypes::Double;
+      field.isSigned = field.type == QgsHanaDataType::SmallInt || field.type == QgsHanaDataType::Integer ||
+                       field.type == QgsHanaDataType::BigInt || field.type == QgsHanaDataType::Decimal ||
+                       field.type == QgsHanaDataType::Numeric || field.type == QgsHanaDataType::Real ||
+                       field.type == QgsHanaDataType::Float || field.type == QgsHanaDataType::Double;
       QString isNullable = rsColumns->getString( 18/*IS_NULLABLE*/ );
       field.isNullable = ( isNullable == QLatin1String( "YES" ) || isNullable == QLatin1String( "TRUE" ) );
       field.isAutoIncrement = isColumnAutoIncrement( field.name );
       field.isUnique = isColumnUnique( field.name );
-      if ( field.isGeometry() )
+      if ( field.type == QgsHanaDataType::Geometry )
         field.srid = getColumnSrid( schemaName, tableName, field.name );
       field.comment = rsColumns->getString( 12/*REMARKS*/ );
 
@@ -911,10 +926,10 @@ QStringList QgsHanaConnection::getPrimaryKeyCandidates( const QgsHanaLayerProper
   QgsHanaResultSetRef rsColumns = getColumns( layerProperty.schemaName, layerProperty.tableName, QStringLiteral( "%" ) );
   while ( rsColumns->next() )
   {
-    int dataType = rsColumns->getValue( 5/*DATA_TYPE */ ).toInt();
-    // We exclude GEOMETRY and LOB columns
-    if ( dataType == 29812 /* GEOMETRY TYPE */ || dataType == SQLDataTypes::LongVarBinary ||
-         dataType == SQLDataTypes::LongVarChar || dataType == SQLDataTypes::WLongVarChar )
+    QgsHanaDataType dataType = QgsHanaDataTypeUtils::fromInt( rsColumns->getValue( 5/*DATA_TYPE */ ).toInt() );
+    // We exclude ST_GEOMETRY, REAL_VECTOR and LOB columns
+    if ( dataType == QgsHanaDataType::Geometry || dataType == QgsHanaDataType::RealVector ||
+         dataType == QgsHanaDataType::LongVarBinary || dataType == QgsHanaDataType::LongVarChar || dataType == QgsHanaDataType::WLongVarChar )
       continue;
     ret << rsColumns->getValue( 4/*COLUMN_NAME */ ).toString();
   }
@@ -1075,7 +1090,7 @@ PreparedStatementRef QgsHanaConnection::createPreparedStatement( const QString &
   PreparedStatementRef stmt = mConnection->prepareStatement( QgsHanaUtils::toUtf16( sql ) );
   if ( !args.isEmpty() )
   {
-    for ( unsigned short i = 1; i <= args.size(); ++i )
+    for ( int i = 1; i <= args.size(); ++i )
     {
       const QVariant &value = args.at( i - 1 );
       switch ( value.userType() )
