@@ -25,6 +25,7 @@ from qgis.core import (
     QgsProfileSnapContext,
     QgsProviderRegistry,
     QgsUnitTypes,
+    QgsCoordinateReferenceSystem
 )
 import unittest
 from qgis.testing import start_app, QgisTestCase
@@ -76,97 +77,97 @@ class TestQgsPointCloudLayerProfileGenerator(QgisTestCase):
                           0.7: 2330.6, 0.9: 2332.7, 1.0: 2325.4, 1.1: 2325.6, 1.2: 2325.6})
 
         self.assertCountEqual([g.asWkt(1) for g in results.asGeometries()],
-                              ['PointZ (515389.1 4918366.7 2326.1)', 'PointZ (515389.1 4918366.6 2325.6)',
-                               'PointZ (515389 4918366.6 2325.3)', 'PointZ (515388.2 4918366.6 2325.2)',
-                               'PointZ (515388.3 4918366.7 2325.1)', 'PointZ (515387.9 4918366.7 2325.2)',
-                               'PointZ (515388.7 4918366.6 2330.5)', 'PointZ (515388.6 4918366.6 2331.2)',
-                               'PointZ (515388.9 4918366.6 2332.7)', 'PointZ (515388.9 4918366.7 2332.7)',
-                               'PointZ (515388.6 4918366.6 2331.4)', 'PointZ (515388.2 4918366.7 2332.2)',
-                               'PointZ (515388.2 4918366.7 2332.6)', 'PointZ (515388.2 4918366.6 2335)',
-                               'PointZ (515388.6 4918366.6 2334.6)', 'PointZ (515389.1 4918366.6 2326.1)',
-                               'PointZ (515389.1 4918366.6 2325.4)', 'PointZ (515389.1 4918366.6 2325.5)',
-                               'PointZ (515389 4918366.6 2325.2)', 'PointZ (515388.3 4918366.6 2325.1)',
-                               'PointZ (515388.6 4918366.6 2330.9)', 'PointZ (515388.7 4918366.6 2330.5)',
-                               'PointZ (515388.6 4918366.6 2330.4)', 'PointZ (515389.1 4918366.6 2325.5)',
-                               'PointZ (515389.1 4918366.6 2325.9)', 'PointZ (515389.1 4918366.6 2325.8)',
-                               'PointZ (515389.1 4918366.7 2325.6)', 'PointZ (515389.1 4918366.6 2325.4)',
-                               'PointZ (515389.1 4918366.7 2325.2)', 'PointZ (515389.1 4918366.6 2326)',
-                               'PointZ (515389.1 4918366.6 2326)', 'PointZ (515389.1 4918366.6 2325.4)',
-                               'PointZ (515389.1 4918366.7 2325.3)', 'PointZ (515389 4918366.6 2325.3)',
-                               'PointZ (515389.1 4918366.7 2325.2)', 'PointZ (515389 4918366.6 2325.4)',
-                               'PointZ (515389 4918366.6 2325.4)', 'PointZ (515389 4918366.7 2325.2)',
-                               'PointZ (515389 4918366.7 2325.4)', 'PointZ (515388.6 4918366.6 2325.2)',
-                               'PointZ (515388.6 4918366.7 2325.2)', 'PointZ (515388.5 4918366.7 2325.2)',
-                               'PointZ (515388.5 4918366.7 2325.2)', 'PointZ (515388.4 4918366.6 2325.1)',
-                               'PointZ (515388.3 4918366.6 2325.1)', 'PointZ (515388.3 4918366.7 2325.1)',
-                               'PointZ (515388.2 4918366.6 2325.1)', 'PointZ (515388.2 4918366.6 2325.2)',
-                               'PointZ (515388.2 4918366.6 2325.2)', 'PointZ (515388.2 4918366.7 2325.2)',
-                               'PointZ (515388.1 4918366.6 2325.2)', 'PointZ (515388.1 4918366.6 2325.2)',
-                               'PointZ (515388 4918366.7 2325.2)', 'PointZ (515388 4918366.6 2325.1)',
-                               'PointZ (515388 4918366.6 2325.2)', 'PointZ (515388.7 4918366.6 2330.6)',
-                               'PointZ (515388.7 4918366.6 2330.5)', 'PointZ (515388.6 4918366.7 2331)',
-                               'PointZ (515388.7 4918366.7 2330.9)', 'PointZ (515388.6 4918366.6 2330.9)',
-                               'PointZ (515388.6 4918366.6 2330.8)', 'PointZ (515388.7 4918366.7 2330.7)',
-                               'PointZ (515388.6 4918366.7 2330.6)', 'PointZ (515389.1 4918366.6 2325.5)',
-                               'PointZ (515389.1 4918366.6 2325.5)', 'PointZ (515389.1 4918366.7 2325.5)',
-                               'PointZ (515389.1 4918366.7 2325.2)', 'PointZ (515389.1 4918366.6 2325.4)',
-                               'PointZ (515389.1 4918366.7 2325.2)', 'PointZ (515389.1 4918366.7 2325.5)',
-                               'PointZ (515389.1 4918366.6 2325.4)', 'PointZ (515389.1 4918366.7 2325.3)',
-                               'PointZ (515389.1 4918366.7 2325.2)', 'PointZ (515389.1 4918366.7 2325.2)',
-                               'PointZ (515389.1 4918366.6 2325.3)', 'PointZ (515389.1 4918366.7 2325.4)',
-                               'PointZ (515389.1 4918366.6 2325.3)', 'PointZ (515389 4918366.7 2325.3)',
-                               'PointZ (515389 4918366.7 2325.3)', 'PointZ (515389.1 4918366.7 2325.3)',
-                               'PointZ (515389 4918366.7 2325.4)', 'PointZ (515389 4918366.7 2325.3)',
-                               'PointZ (515389 4918366.6 2325.2)', 'PointZ (515389 4918366.6 2325.4)',
-                               'PointZ (515389 4918366.6 2325.3)', 'PointZ (515389 4918366.6 2325.3)',
-                               'PointZ (515389 4918366.7 2325.2)', 'PointZ (515389 4918366.7 2325.2)',
-                               'PointZ (515389 4918366.6 2325.4)', 'PointZ (515389 4918366.6 2325.3)',
-                               'PointZ (515389 4918366.7 2325.3)', 'PointZ (515389 4918366.7 2325.2)',
-                               'PointZ (515389 4918366.6 2325.3)', 'PointZ (515389 4918366.7 2325.3)',
-                               'PointZ (515389 4918366.7 2325.2)', 'PointZ (515388.6 4918366.7 2325.2)',
-                               'PointZ (515388.6 4918366.7 2325.2)', 'PointZ (515388.5 4918366.7 2325.1)',
-                               'PointZ (515388.4 4918366.6 2325.1)', 'PointZ (515388.4 4918366.7 2325.1)',
-                               'PointZ (515388.4 4918366.6 2325.1)', 'PointZ (515388.4 4918366.7 2325.1)',
-                               'PointZ (515388.4 4918366.6 2325.1)', 'PointZ (515388.3 4918366.6 2325.1)',
-                               'PointZ (515388.3 4918366.6 2325.1)', 'PointZ (515388.2 4918366.7 2325.1)',
-                               'PointZ (515388.2 4918366.6 2325.2)', 'PointZ (515388.2 4918366.7 2325.2)',
-                               'PointZ (515388.3 4918366.7 2325.1)', 'PointZ (515388.1 4918366.7 2325.2)',
-                               'PointZ (515388.1 4918366.7 2325.1)', 'PointZ (515388.1 4918366.7 2325.1)',
-                               'PointZ (515388.1 4918366.7 2325.2)', 'PointZ (515388 4918366.6 2325.1)',
-                               'PointZ (515389.1 4918366.6 2325.8)', 'PointZ (515389.1 4918366.6 2325.8)',
-                               'PointZ (515389.1 4918366.7 2325.8)', 'PointZ (515389.1 4918366.7 2325.6)',
-                               'PointZ (515389.1 4918366.6 2325.5)', 'PointZ (515389.1 4918366.6 2325.9)',
-                               'PointZ (515389.1 4918366.6 2325.9)', 'PointZ (515389.1 4918366.6 2325.9)',
-                               'PointZ (515389.2 4918366.7 2325.6)', 'PointZ (515389.1 4918366.7 2325.8)',
-                               'PointZ (515389.1 4918366.7 2325.5)', 'PointZ (515389.1 4918366.6 2326)',
-                               'PointZ (515389.1 4918366.6 2326)', 'PointZ (515389.1 4918366.7 2326)',
-                               'PointZ (515388.7 4918366.6 2330.6)', 'PointZ (515388.7 4918366.6 2330.6)',
-                               'PointZ (515388.7 4918366.6 2330.5)', 'PointZ (515388.7 4918366.7 2331)',
-                               'PointZ (515388.7 4918366.7 2330.7)', 'PointZ (515388.7 4918366.7 2330.8)',
-                               'PointZ (515388.7 4918366.7 2330.6)', 'PointZ (515388.7 4918366.7 2330.9)',
-                               'PointZ (515388.7 4918366.7 2330.8)', 'PointZ (515388.6 4918366.6 2331.1)',
-                               'PointZ (515388.6 4918366.7 2331.3)', 'PointZ (515388.6 4918366.7 2331.3)',
-                               'PointZ (515388.3 4918366.6 2334.7)', 'PointZ (515388.6 4918366.6 2331.1)',
-                               'PointZ (515388.6 4918366.6 2331)', 'PointZ (515388.6 4918366.7 2331)',
-                               'PointZ (515388.6 4918366.6 2331.3)', 'PointZ (515388.6 4918366.7 2331.2)',
-                               'PointZ (515388.6 4918366.6 2331.3)', 'PointZ (515388.6 4918366.7 2331.4)',
-                               'PointZ (515388.2 4918366.6 2332.4)', 'PointZ (515388.2 4918366.7 2332.2)',
-                               'PointZ (515388.2 4918366.7 2332.3)', 'PointZ (515388.2 4918366.7 2332.7)',
-                               'PointZ (515388.2 4918366.7 2332.7)', 'PointZ (515388.2 4918366.7 2332.7)',
-                               'PointZ (515388.2 4918366.7 2332.6)', 'PointZ (515388.2 4918366.6 2332.5)',
-                               'PointZ (515388.2 4918366.6 2332.5)', 'PointZ (515388.3 4918366.6 2334.7)',
-                               'PointZ (515388.3 4918366.7 2334.7)', 'PointZ (515388.2 4918366.7 2335.1)',
-                               'PointZ (515388.6 4918366.6 2331.2)', 'PointZ (515388.6 4918366.6 2331.1)',
-                               'PointZ (515388.6 4918366.7 2331.1)', 'PointZ (515388.6 4918366.7 2331.1)',
-                               'PointZ (515388.6 4918366.6 2331.3)', 'PointZ (515388.6 4918366.7 2331.3)',
-                               'PointZ (515388.6 4918366.6 2331.1)', 'PointZ (515388.6 4918366.7 2331.3)',
-                               'PointZ (515388.6 4918366.7 2331.2)', 'PointZ (515388.6 4918366.7 2331.4)',
-                               'PointZ (515388.6 4918366.7 2331.4)', 'PointZ (515388.2 4918366.6 2332.3)',
-                               'PointZ (515388.2 4918366.7 2332.4)', 'PointZ (515388.2 4918366.7 2332.4)',
-                               'PointZ (515388.2 4918366.7 2332.7)', 'PointZ (515388.2 4918366.6 2332.6)',
-                               'PointZ (515388.2 4918366.7 2332.6)', 'PointZ (515388.2 4918366.6 2332.5)',
-                               'PointZ (515388.2 4918366.6 2332.5)', 'PointZ (515388.2 4918366.7 2332.4)',
-                               'PointZ (515388.2 4918366.7 2332.6)', 'PointZ (515388.3 4918366.7 2334.7)'])
+                              ['Point Z (515389.1 4918366.7 2326.1)', 'Point Z (515389.1 4918366.6 2325.6)',
+                               'Point Z (515389 4918366.6 2325.3)', 'Point Z (515388.2 4918366.6 2325.2)',
+                               'Point Z (515388.3 4918366.7 2325.1)', 'Point Z (515387.9 4918366.7 2325.2)',
+                               'Point Z (515388.7 4918366.6 2330.5)', 'Point Z (515388.6 4918366.6 2331.2)',
+                               'Point Z (515388.9 4918366.6 2332.7)', 'Point Z (515388.9 4918366.7 2332.7)',
+                               'Point Z (515388.6 4918366.6 2331.4)', 'Point Z (515388.2 4918366.7 2332.2)',
+                               'Point Z (515388.2 4918366.7 2332.6)', 'Point Z (515388.2 4918366.6 2335)',
+                               'Point Z (515388.6 4918366.6 2334.6)', 'Point Z (515389.1 4918366.6 2326.1)',
+                               'Point Z (515389.1 4918366.6 2325.4)', 'Point Z (515389.1 4918366.6 2325.5)',
+                               'Point Z (515389 4918366.6 2325.2)', 'Point Z (515388.3 4918366.6 2325.1)',
+                               'Point Z (515388.6 4918366.6 2330.9)', 'Point Z (515388.7 4918366.6 2330.5)',
+                               'Point Z (515388.6 4918366.6 2330.4)', 'Point Z (515389.1 4918366.6 2325.5)',
+                               'Point Z (515389.1 4918366.6 2325.9)', 'Point Z (515389.1 4918366.6 2325.8)',
+                               'Point Z (515389.1 4918366.7 2325.6)', 'Point Z (515389.1 4918366.6 2325.4)',
+                               'Point Z (515389.1 4918366.7 2325.2)', 'Point Z (515389.1 4918366.6 2326)',
+                               'Point Z (515389.1 4918366.6 2326)', 'Point Z (515389.1 4918366.6 2325.4)',
+                               'Point Z (515389.1 4918366.7 2325.3)', 'Point Z (515389 4918366.6 2325.3)',
+                               'Point Z (515389.1 4918366.7 2325.2)', 'Point Z (515389 4918366.6 2325.4)',
+                               'Point Z (515389 4918366.6 2325.4)', 'Point Z (515389 4918366.7 2325.2)',
+                               'Point Z (515389 4918366.7 2325.4)', 'Point Z (515388.6 4918366.6 2325.2)',
+                               'Point Z (515388.6 4918366.7 2325.2)', 'Point Z (515388.5 4918366.7 2325.2)',
+                               'Point Z (515388.5 4918366.7 2325.2)', 'Point Z (515388.4 4918366.6 2325.1)',
+                               'Point Z (515388.3 4918366.6 2325.1)', 'Point Z (515388.3 4918366.7 2325.1)',
+                               'Point Z (515388.2 4918366.6 2325.1)', 'Point Z (515388.2 4918366.6 2325.2)',
+                               'Point Z (515388.2 4918366.6 2325.2)', 'Point Z (515388.2 4918366.7 2325.2)',
+                               'Point Z (515388.1 4918366.6 2325.2)', 'Point Z (515388.1 4918366.6 2325.2)',
+                               'Point Z (515388 4918366.7 2325.2)', 'Point Z (515388 4918366.6 2325.1)',
+                               'Point Z (515388 4918366.6 2325.2)', 'Point Z (515388.7 4918366.6 2330.6)',
+                               'Point Z (515388.7 4918366.6 2330.5)', 'Point Z (515388.6 4918366.7 2331)',
+                               'Point Z (515388.7 4918366.7 2330.9)', 'Point Z (515388.6 4918366.6 2330.9)',
+                               'Point Z (515388.6 4918366.6 2330.8)', 'Point Z (515388.7 4918366.7 2330.7)',
+                               'Point Z (515388.6 4918366.7 2330.6)', 'Point Z (515389.1 4918366.6 2325.5)',
+                               'Point Z (515389.1 4918366.6 2325.5)', 'Point Z (515389.1 4918366.7 2325.5)',
+                               'Point Z (515389.1 4918366.7 2325.2)', 'Point Z (515389.1 4918366.6 2325.4)',
+                               'Point Z (515389.1 4918366.7 2325.2)', 'Point Z (515389.1 4918366.7 2325.5)',
+                               'Point Z (515389.1 4918366.6 2325.4)', 'Point Z (515389.1 4918366.7 2325.3)',
+                               'Point Z (515389.1 4918366.7 2325.2)', 'Point Z (515389.1 4918366.7 2325.2)',
+                               'Point Z (515389.1 4918366.6 2325.3)', 'Point Z (515389.1 4918366.7 2325.4)',
+                               'Point Z (515389.1 4918366.6 2325.3)', 'Point Z (515389 4918366.7 2325.3)',
+                               'Point Z (515389 4918366.7 2325.3)', 'Point Z (515389.1 4918366.7 2325.3)',
+                               'Point Z (515389 4918366.7 2325.4)', 'Point Z (515389 4918366.7 2325.3)',
+                               'Point Z (515389 4918366.6 2325.2)', 'Point Z (515389 4918366.6 2325.4)',
+                               'Point Z (515389 4918366.6 2325.3)', 'Point Z (515389 4918366.6 2325.3)',
+                               'Point Z (515389 4918366.7 2325.2)', 'Point Z (515389 4918366.7 2325.2)',
+                               'Point Z (515389 4918366.6 2325.4)', 'Point Z (515389 4918366.6 2325.3)',
+                               'Point Z (515389 4918366.7 2325.3)', 'Point Z (515389 4918366.7 2325.2)',
+                               'Point Z (515389 4918366.6 2325.3)', 'Point Z (515389 4918366.7 2325.3)',
+                               'Point Z (515389 4918366.7 2325.2)', 'Point Z (515388.6 4918366.7 2325.2)',
+                               'Point Z (515388.6 4918366.7 2325.2)', 'Point Z (515388.5 4918366.7 2325.1)',
+                               'Point Z (515388.4 4918366.6 2325.1)', 'Point Z (515388.4 4918366.7 2325.1)',
+                               'Point Z (515388.4 4918366.6 2325.1)', 'Point Z (515388.4 4918366.7 2325.1)',
+                               'Point Z (515388.4 4918366.6 2325.1)', 'Point Z (515388.3 4918366.6 2325.1)',
+                               'Point Z (515388.3 4918366.6 2325.1)', 'Point Z (515388.2 4918366.7 2325.1)',
+                               'Point Z (515388.2 4918366.6 2325.2)', 'Point Z (515388.2 4918366.7 2325.2)',
+                               'Point Z (515388.3 4918366.7 2325.1)', 'Point Z (515388.1 4918366.7 2325.2)',
+                               'Point Z (515388.1 4918366.7 2325.1)', 'Point Z (515388.1 4918366.7 2325.1)',
+                               'Point Z (515388.1 4918366.7 2325.2)', 'Point Z (515388 4918366.6 2325.1)',
+                               'Point Z (515389.1 4918366.6 2325.8)', 'Point Z (515389.1 4918366.6 2325.8)',
+                               'Point Z (515389.1 4918366.7 2325.8)', 'Point Z (515389.1 4918366.7 2325.6)',
+                               'Point Z (515389.1 4918366.6 2325.5)', 'Point Z (515389.1 4918366.6 2325.9)',
+                               'Point Z (515389.1 4918366.6 2325.9)', 'Point Z (515389.1 4918366.6 2325.9)',
+                               'Point Z (515389.2 4918366.7 2325.6)', 'Point Z (515389.1 4918366.7 2325.8)',
+                               'Point Z (515389.1 4918366.7 2325.5)', 'Point Z (515389.1 4918366.6 2326)',
+                               'Point Z (515389.1 4918366.6 2326)', 'Point Z (515389.1 4918366.7 2326)',
+                               'Point Z (515388.7 4918366.6 2330.6)', 'Point Z (515388.7 4918366.6 2330.6)',
+                               'Point Z (515388.7 4918366.6 2330.5)', 'Point Z (515388.7 4918366.7 2331)',
+                               'Point Z (515388.7 4918366.7 2330.7)', 'Point Z (515388.7 4918366.7 2330.8)',
+                               'Point Z (515388.7 4918366.7 2330.6)', 'Point Z (515388.7 4918366.7 2330.9)',
+                               'Point Z (515388.7 4918366.7 2330.8)', 'Point Z (515388.6 4918366.6 2331.1)',
+                               'Point Z (515388.6 4918366.7 2331.3)', 'Point Z (515388.6 4918366.7 2331.3)',
+                               'Point Z (515388.3 4918366.6 2334.7)', 'Point Z (515388.6 4918366.6 2331.1)',
+                               'Point Z (515388.6 4918366.6 2331)', 'Point Z (515388.6 4918366.7 2331)',
+                               'Point Z (515388.6 4918366.6 2331.3)', 'Point Z (515388.6 4918366.7 2331.2)',
+                               'Point Z (515388.6 4918366.6 2331.3)', 'Point Z (515388.6 4918366.7 2331.4)',
+                               'Point Z (515388.2 4918366.6 2332.4)', 'Point Z (515388.2 4918366.7 2332.2)',
+                               'Point Z (515388.2 4918366.7 2332.3)', 'Point Z (515388.2 4918366.7 2332.7)',
+                               'Point Z (515388.2 4918366.7 2332.7)', 'Point Z (515388.2 4918366.7 2332.7)',
+                               'Point Z (515388.2 4918366.7 2332.6)', 'Point Z (515388.2 4918366.6 2332.5)',
+                               'Point Z (515388.2 4918366.6 2332.5)', 'Point Z (515388.3 4918366.6 2334.7)',
+                               'Point Z (515388.3 4918366.7 2334.7)', 'Point Z (515388.2 4918366.7 2335.1)',
+                               'Point Z (515388.6 4918366.6 2331.2)', 'Point Z (515388.6 4918366.6 2331.1)',
+                               'Point Z (515388.6 4918366.7 2331.1)', 'Point Z (515388.6 4918366.7 2331.1)',
+                               'Point Z (515388.6 4918366.6 2331.3)', 'Point Z (515388.6 4918366.7 2331.3)',
+                               'Point Z (515388.6 4918366.6 2331.1)', 'Point Z (515388.6 4918366.7 2331.3)',
+                               'Point Z (515388.6 4918366.7 2331.2)', 'Point Z (515388.6 4918366.7 2331.4)',
+                               'Point Z (515388.6 4918366.7 2331.4)', 'Point Z (515388.2 4918366.6 2332.3)',
+                               'Point Z (515388.2 4918366.7 2332.4)', 'Point Z (515388.2 4918366.7 2332.4)',
+                               'Point Z (515388.2 4918366.7 2332.7)', 'Point Z (515388.2 4918366.6 2332.6)',
+                               'Point Z (515388.2 4918366.7 2332.6)', 'Point Z (515388.2 4918366.6 2332.5)',
+                               'Point Z (515388.2 4918366.6 2332.5)', 'Point Z (515388.2 4918366.7 2332.4)',
+                               'Point Z (515388.2 4918366.7 2332.6)', 'Point Z (515388.3 4918366.7 2334.7)'])
         self.assertAlmostEqual(results.zRange().lower(), 2325.1325, 2)
         self.assertAlmostEqual(results.zRange().upper(), 2335.0755, 2)
 
@@ -174,10 +175,10 @@ class TestQgsPointCloudLayerProfileGenerator(QgisTestCase):
         self.assertEqual(len(features), 182)
         self.assertEqual(features[0].layerIdentifier, pcl.id())
         self.assertEqual(features[0].geometry.asWkt(1),
-                         'PointZ (515389.1 4918366.7 2326.1)')
+                         'Point Z (515389.1 4918366.7 2326.1)')
         self.assertEqual(features[-1].layerIdentifier, pcl.id())
         self.assertEqual(features[-1].geometry.asWkt(1),
-                         'PointZ (515388.3 4918366.7 2334.7)')
+                         'Point Z (515388.3 4918366.7 2334.7)')
 
         features = results.asFeatures(Qgis.ProfileExportType.Profile2D)
         self.assertEqual(len(features), 182)
@@ -193,8 +194,8 @@ class TestQgsPointCloudLayerProfileGenerator(QgisTestCase):
         self.assertEqual(features[0].layerIdentifier, pcl.id())
         self.assertAlmostEqual(features[0].attributes['distance'], 1.129138, 2)
         self.assertAlmostEqual(features[0].attributes['elevation'], 2326.052, 2)
-        self.assertEqual(features[0].geometry.asWkt(1), 'PointZ (515389.1 4918366.7 2326.1)')
-        self.assertEqual(features[-1].geometry.asWkt(1), 'PointZ (515388.3 4918366.7 2334.7)')
+        self.assertEqual(features[0].geometry.asWkt(1), 'Point Z (515389.1 4918366.7 2326.1)')
+        self.assertEqual(features[-1].geometry.asWkt(1), 'Point Z (515388.3 4918366.7 2334.7)')
         self.assertAlmostEqual(features[-1].attributes['distance'], 0.319292, 2)
         self.assertAlmostEqual(features[-1].attributes['elevation'], 2334.69125, 2)
 
@@ -444,6 +445,68 @@ class TestQgsPointCloudLayerProfileGenerator(QgisTestCase):
                              allowed_mismatch=20)
         )
 
+    @unittest.skipIf('ept' not in QgsProviderRegistry.instance().providerList(), 'EPT provider not available')
+    def test_vertical_transformation_4979_to_4985(self):
+        pcl = QgsPointCloudLayer(
+            self.get_test_data_path('point_clouds/ept/rgb16/ept.json').as_posix(), 'test', 'ept')
+
+        self.assertTrue(pcl.isValid())
+        pcl.setCrs(QgsCoordinateReferenceSystem('EPSG:4979'))
+        self.assertEqual(pcl.crs3D().authid(), 'EPSG:4979')
+
+        pcl.elevationProperties().setMaximumScreenError(30)
+        pcl.elevationProperties().setMaximumScreenErrorUnit(QgsUnitTypes.RenderUnit.RenderMillimeters)
+
+        curve = QgsLineString()
+        curve.fromWkt(
+            'LineString (7.37810825606327025 2.69442638932088574, 7.44718273878368908 2.71100426469523947)')
+        req = QgsProfileRequest(curve)
+        req.setCrs(QgsCoordinateReferenceSystem('EPSG:4985'))
+
+        req.setTolerance(0.005)
+        generator = pcl.createProfileGenerator(req)
+
+        context = QgsProfileGenerationContext()
+        context.setMapUnitsPerDistancePixel(0.50)
+
+        self.assertTrue(generator.generateProfile(context))
+        results = generator.takeResults()
+        self.assertEqual(self.round_dict(results.distanceToHeightMap(), 3),
+                         {0.013: -5.409, 0.064: -5.41})
+
+        self.assertCountEqual([g.asWkt(3) for g in results.asGeometries()],
+                              ['Point Z (7.39 2.7 -5.409)', 'Point Z (7.44 2.71 -5.41)'])
+        self.assertAlmostEqual(results.zRange().lower(), -5.40999, 4)
+        self.assertAlmostEqual(results.zRange().upper(), -5.40920, 4)
+
+        features = results.asFeatures(Qgis.ProfileExportType.Features3D)
+        self.assertEqual(len(features), 2)
+        self.assertEqual(features[0].layerIdentifier, pcl.id())
+        self.assertEqual(features[0].geometry.asWkt(3),
+                         'Point Z (7.39 2.7 -5.409)')
+        self.assertEqual(features[-1].layerIdentifier, pcl.id())
+        self.assertEqual(features[-1].geometry.asWkt(3),
+                         'Point Z (7.44 2.71 -5.41)')
+
+        features = results.asFeatures(Qgis.ProfileExportType.Profile2D)
+        self.assertEqual(len(features), 2)
+        self.assertEqual(features[0].layerIdentifier, pcl.id())
+        self.assertEqual(features[0].geometry.asWkt(3),
+                         'Point (0.013 -5.409)')
+        self.assertEqual(features[-1].layerIdentifier, pcl.id())
+        self.assertEqual(features[-1].geometry.asWkt(3),
+                         'Point (0.064 -5.41)')
+
+        features = results.asFeatures(Qgis.ProfileExportType.DistanceVsElevationTable)
+        self.assertEqual(len(features), 2)
+        self.assertEqual(features[0].layerIdentifier, pcl.id())
+        self.assertAlmostEqual(features[0].attributes['distance'], 0.012704944, 4)
+        self.assertAlmostEqual(features[0].attributes['elevation'], -5.409209, 4)
+        self.assertEqual(features[0].geometry.asWkt(3), 'Point Z (7.39 2.7 -5.409)')
+        self.assertEqual(features[-1].geometry.asWkt(3), 'Point Z (7.44 2.71 -5.41)')
+        self.assertAlmostEqual(features[-1].attributes['distance'], 0.063658039178, 4)
+        self.assertAlmostEqual(features[-1].attributes['elevation'], -5.409997397, 4)
+
 
 if __name__ == '__main__':
     unittest.main()
