@@ -11,7 +11,7 @@ __author__ = 'Nyall Dawson'
 __date__ = '12/05/2020'
 __copyright__ = 'Copyright 2020, The QGIS Project'
 
-from qgis.PyQt.QtCore import QT_VERSION_STR
+from qgis.PyQt.QtCore import QT_VERSION_STR, QSizeF
 from qgis.core import (
     Qgis,
     QgsFontUtils,
@@ -20,6 +20,7 @@ from qgis.core import (
     QgsTextCharacterFormat,
     QgsTextDocument,
     QgsTextFragment,
+    QgsTextFormat
 )
 import unittest
 from qgis.testing import start_app, QgisTestCase
@@ -82,9 +83,10 @@ class TestQgsTextDocument(QgisTestCase):
         self.assertEqual(doc[0][5].text(), 'd')
 
     def testFromHtml(self):
-        doc = QgsTextDocument.fromHtml(['abc<div style="color: red"><b style="text-decoration: underline; font-style: italic; font-size: 15pt; font-family: Serif">def</b> ghi<div>jkl</div></div>', 'b c d', 'e'])
+        doc = QgsTextDocument.fromHtml(['abc<div style="color: red; text-align: right"><b style="text-decoration: underline; font-style: italic; font-size: 15pt; font-family: Serif">def</b> ghi<div>jkl</div></div>', 'b c d', 'e'])
         self.assertEqual(len(doc), 5)
         self.assertEqual(len(doc[0]), 1)
+        self.assertFalse(doc[0].blockFormat().hasHorizontalAlignmentSet())
         self.assertEqual(doc[0][0].text(), 'abc')
         self.assertEqual(doc[0][0].characterFormat().underline(), QgsTextCharacterFormat.BooleanValue.NotSet)
         self.assertEqual(doc[0][0].characterFormat().italic(), QgsTextCharacterFormat.BooleanValue.NotSet)
@@ -97,6 +99,8 @@ class TestQgsTextDocument(QgisTestCase):
         self.assertEqual(doc[1][0].text(), 'def')
         self.assertEqual(doc[1][0].characterFormat().underline(), QgsTextCharacterFormat.BooleanValue.SetTrue)
         self.assertEqual(doc[1][0].characterFormat().italic(), QgsTextCharacterFormat.BooleanValue.SetTrue)
+        self.assertTrue(doc[1].blockFormat().hasHorizontalAlignmentSet())
+        self.assertEqual(doc[1].blockFormat().horizontalAlignment(), Qgis.TextHorizontalAlignment.Right)
         if int(QT_VERSION_STR.split('.')[0]) >= 6:
             self.assertEqual(doc[1][0].characterFormat().fontWeight(), 700)
         else:
@@ -196,6 +200,30 @@ class TestQgsTextDocument(QgisTestCase):
         self.assertTrue(doc[1][1].isTab())
         self.assertEqual(doc[1][2].text(), 'cdcd')
 
+    def testFromTextAndFormat(self):
+        format = QgsTextFormat()
+        format.setAllowHtmlFormatting(False)
+        doc = QgsTextDocument.fromTextAndFormat(['abc <b>def</b>'], format)
+        self.assertEqual(len(doc), 1)
+        self.assertEqual(len(doc[0]), 1)
+        self.assertEqual(doc[0][0].text(), 'abc <b>def</b>')
+
+        # as html
+        format.setAllowHtmlFormatting(True)
+        doc = QgsTextDocument.fromTextAndFormat(['abc <b>def</b>'], format)
+        self.assertEqual(len(doc), 1)
+        self.assertEqual(len(doc[0]), 2)
+        self.assertEqual(doc[0][0].text(), 'abc ')
+        self.assertEqual(doc[0][1].text(), 'def')
+
+        # with capitalization option
+        format.setCapitalization(Qgis.Capitalization.AllUppercase)
+        format.setAllowHtmlFormatting(False)
+        doc = QgsTextDocument.fromTextAndFormat(['abc def'], format)
+        self.assertEqual(len(doc), 1)
+        self.assertEqual(len(doc[0]), 1)
+        self.assertEqual(doc[0][0].text(), 'ABC DEF')
+
     def testFromHtmlVerticalAlignment(self):
         doc = QgsTextDocument.fromHtml(['abc<div style="color: red"><sub>def<b>extra</b></sub> ghi</div><sup>sup</sup><span style="vertical-align: sub">css</span>'])
         self.assertEqual(len(doc), 3)
@@ -219,6 +247,22 @@ class TestQgsTextDocument(QgisTestCase):
         self.assertTrue(doc[2][1].characterFormat().hasVerticalAlignmentSet())
         self.assertEqual(doc[2][1].characterFormat().verticalAlignment(), Qgis.TextCharacterVerticalAlignment.SubScript)
 
+    def testImage(self):
+        doc = QgsTextDocument.fromHtml([
+            'abc<img src="qgis.jpg" width=40 height=60><i>extra</i>'])
+        self.assertEqual(len(doc), 1)
+        self.assertEqual(len(doc[0]), 3)
+        self.assertEqual(doc[0][0].text(), 'abc')
+        self.assertFalse(doc[0][0].isImage())
+        self.assertFalse(doc[0][0].characterFormat().imagePath())
+        self.assertTrue(doc[0][1].isImage())
+        self.assertFalse(doc[0][1].text())
+        self.assertEqual(doc[0][1].characterFormat().imagePath(), 'qgis.jpg')
+        self.assertEqual(doc[0][1].characterFormat().imageSize(), QSizeF(40, 60))
+        self.assertEqual(doc[0][2].text(), 'extra')
+        self.assertFalse(doc[0][2].isImage())
+        self.assertTrue(doc[0][2].characterFormat().italic())
+
     def testAppend(self):
         doc = QgsTextDocument()
         self.assertEqual(len(doc), 0)
@@ -228,6 +272,39 @@ class TestQgsTextDocument(QgisTestCase):
         self.assertEqual(len(doc), 1)
         self.assertEqual(len(doc[0]), 0)
 
+    def testInsert(self):
+        doc = QgsTextDocument()
+        self.assertEqual(len(doc), 0)
+
+        with self.assertRaises(IndexError):
+            doc.insert(-1, QgsTextBlock(QgsTextFragment('a')))
+        with self.assertRaises(IndexError):
+            doc.insert(1, QgsTextBlock(QgsTextFragment('a')))
+        self.assertEqual(len(doc), 0)
+        doc.insert(0, QgsTextBlock(QgsTextFragment('a')))
+        self.assertEqual(len(doc), 1)
+        self.assertEqual(doc[0][0].text(), 'a')
+
+        doc.insert(0, QgsTextBlock(QgsTextFragment('b')))
+        self.assertEqual(len(doc), 2)
+        self.assertEqual(doc[0][0].text(), 'b')
+        self.assertEqual(doc[1][0].text(), 'a')
+
+        doc.insert(1, QgsTextBlock(QgsTextFragment('c')))
+        self.assertEqual(len(doc), 3)
+        self.assertEqual(doc[0][0].text(), 'b')
+        self.assertEqual(doc[1][0].text(), 'c')
+        self.assertEqual(doc[2][0].text(), 'a')
+
+        with self.assertRaises(IndexError):
+            doc.insert(4, QgsTextBlock(QgsTextFragment('d')))
+        doc.insert(3, QgsTextBlock(QgsTextFragment('d')))
+        self.assertEqual(len(doc), 4)
+        self.assertEqual(doc[0][0].text(), 'b')
+        self.assertEqual(doc[1][0].text(), 'c')
+        self.assertEqual(doc[2][0].text(), 'a')
+        self.assertEqual(doc[3][0].text(), 'd')
+
     def testAt(self):
         doc = QgsTextDocument()
         self.assertEqual(len(doc), 0)
